import { transition, trigger } from '@angular/animations';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Component, EventEmitter, Input, Output, ViewEncapsulation, HostBinding, HostListener, NgModule } from '@angular/core';
import { fadeOut } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtCalloutComponent {
    constructor() {
        this.title = '';
        this.color = 'primary';
        this.size = 'medium';
        /**
         * 定义此属性是为了避免内部的动态样式 class 和外部设置的值起冲突，所以用此属性接收外部的值并且合并 class 来避免这个问题。
         *
         * TODO: 官方会在 Ivy Renderer 中修复这个问题，预计会在 Angular 9.0。
         * 有关此问题的Issue: https://github.com/angular/angular/issues/7289
         * @deprecated > 0.6.0
         */
        this.class = '';
        this._closed = false;
        this.close = new EventEmitter();
        this.fadeOut = false;
    }
    /**
     * @return {?}
     */
    get closed() { return this._closed; }
    /**
     * @param {?} value
     * @return {?}
     */
    set closable(value) { this._closable = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get closable() { return this._closable; }
    /**
     * @param {?} event
     * @return {?}
     */
    onAnimationDone(event) {
        if (event.toState) {
            this._closed = true;
        }
    }
    /**
     * @return {?}
     */
    _close() {
        if (!this.fadeOut) {
            this.fadeOut = true;
            this.close.emit();
        }
    }
}
NtCalloutComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-callout',
                template: `
    <h3 *ngIf="title">{{title}}</h3>
    <ng-content></ng-content>
    <button *ngIf="this.closable" (click)="_close()" class="close-button" type="button">
      <span aria-hidden="true">&times;</span>
    </button>
  `,
                encapsulation: ViewEncapsulation.None,
                animations: [
                    trigger('fadeOut', [
                        transition('* => true', fadeOut(.3))
                    ])
                ],
                host: {
                    '[class]': '["callout nt-callout", color || "primary", size, class].join(" ")',
                    '[class.closed]': 'closed'
                }
            },] },
];
/** @nocollapse */
NtCalloutComponent.ctorParameters = () => [];
NtCalloutComponent.propDecorators = {
    title: [{ type: Input }],
    color: [{ type: Input }],
    size: [{ type: Input }],
    class: [{ type: Input }],
    closable: [{ type: Input }],
    close: [{ type: Output }],
    fadeOut: [{ type: HostBinding, args: ['@fadeOut',] }],
    onAnimationDone: [{ type: HostListener, args: ['@fadeOut.done', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtCalloutModule {
}
NtCalloutModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtCalloutComponent],
                declarations: [NtCalloutComponent],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtCalloutModule, NtCalloutComponent };
//# sourceMappingURL=callout.js.map
