import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Attribute, ChangeDetectionStrategy, ChangeDetectorRef, Component, EventEmitter, Input, Optional, Output, Self, ViewEncapsulation, ContentChildren, isDevMode, NgZone, NgModule } from '@angular/core';
import { NgControl } from '@angular/forms';
import { defer, merge, Subject } from 'rxjs';
import { startWith, switchMap, take, takeUntil } from 'rxjs/operators';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
let uniqueId = 0;
class NtCheckboxChange {
    /**
     * @param {?} source
     * @param {?} checked
     */
    constructor(source, checked) {
        this.source = source;
        this.checked = checked;
    }
}
class NtCheckboxComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} tabIndex
     * @param {?} ngControl
     */
    constructor(_changeDetectorRef, tabIndex, ngControl) {
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this.id = `nt-checkbox-${uniqueId++}`;
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
        this._indeterminate = false;
        this.change = new EventEmitter();
        // TODO: 考虑合并到 change 事件中
        this.indeterminateChange = new EventEmitter();
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this.tabIndex = parseInt(tabIndex) || 0;
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) { this._value = value; }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) {
        if (value !== this.disabled) {
            this._disabled = coerceBooleanProperty(value);
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    get readonly() { return this._readonly; }
    /**
     * @param {?} value
     * @return {?}
     */
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get checked() { return this._checked; }
    /**
     * @param {?} value
     * @return {?}
     */
    set checked(value) {
        if (value !== this.checked) {
            this._checked = coerceBooleanProperty(value);
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    get indeterminate() { return this._indeterminate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set indeterminate(value) {
        /** @type {?} */
        const changed = value !== this._indeterminate;
        this._indeterminate = value;
        if (changed) {
            this.indeterminateChange.emit(this._indeterminate);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._checked = !!value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onInputClick(event) {
        event.stopPropagation();
        if (!this.disabled) {
            this._checked = !this._checked;
            this._onTouched();
            this._emitChangeEvent();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onInteractionEvent(event) {
        // We always have to stop propagation on the change event.
        // Otherwise the change event, from the input element, will bubble up and
        // emit its event object to the `change` output.
        event.stopPropagation();
    }
    /**
     * @private
     * @return {?}
     */
    _emitChangeEvent() {
        /** @type {?} */
        const event = new NtCheckboxChange(this, this.checked);
        this._onChange(this.checked);
        this.change.emit(event);
    }
}
NtCheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-checkbox',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<label class=\"nt-checkbox-layout\"> <div class=\"nt-checkbox-input-container\"> <input class=\"cdk-visually-hidden\" type=\"checkbox\" [tabIndex]=\"tabIndex\" [checked]=\"checked\" [disabled]=\"disabled\" [readonly]=\"readonly\" (change)=\"_onInteractionEvent($event)\" (click)=\"_onInputClick($event)\"> <div class=\"nt-checkbox-input nt-form-control\"></div> </div> <span class=\"nt-checkbox-label\"> <ng-content></ng-content> </span> </label> ",
                host: {
                    'class': 'nt-checkbox',
                    '[class.nt-checkbox-checked]': 'checked',
                    '[class.nt-checkbox-disabled]': 'disabled',
                    '[class.nt-checkbox-indeterminate]': 'indeterminate',
                }
            },] },
];
/** @nocollapse */
NtCheckboxComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] }
];
NtCheckboxComponent.propDecorators = {
    value: [{ type: Input }],
    disabled: [{ type: Input }],
    readonly: [{ type: Input }],
    checked: [{ type: Input }],
    indeterminate: [{ type: Input }],
    change: [{ type: Output }],
    indeterminateChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function getNtCheckboxGroupNonFunctionValueError() {
    return Error('`compareWith` must be a function.');
}
/** @type {?} */
let uniqueId$1 = 0;
/**
 * @template T
 */
class NtCheckboxGroupComponent extends NtFormFieldControl {
    /**
     * @param {?} _ngZone
     * @param {?} _changeDetectorRef
     * @param {?} ngControl
     */
    constructor(_ngZone, _changeDetectorRef, ngControl) {
        super();
        this._ngZone = _ngZone;
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this.id = `nt-checkbox-group-${uniqueId$1++}`;
        this._destroy = new Subject();
        this._disabled = false;
        this._readonly = false;
        this._required = false;
        this._compareWith = (/**
         * @param {?} o1
         * @param {?} o2
         * @return {?}
         */
        (o1, o2) => o1 === o2);
        this.checkedChanges = defer((/**
         * @return {?}
         */
        () => {
            if (this.checkboxes) {
                return merge(...this.checkboxes.map((/**
                 * @param {?} item
                 * @return {?}
                 */
                item => item.change)));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            () => this.checkedChanges)));
        }));
        /**
         * Emits when the value changes (either due to user input or programmatic change).
         */
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @return {?}
     */
    get required() { return this._required; }
    /**
     * @param {?} value
     * @return {?}
     */
    set required(value) { this._required = coerceBooleanProperty(value); }
    /**
     * @param {?} value
     * @return {?}
     */
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get readonly() { return this._readonly; }
    /**
     * @param {?} fn
     * @return {?}
     */
    set compareWith(fn) {
        if (typeof fn !== 'function') {
            throw getNtCheckboxGroupNonFunctionValueError();
        }
        else {
            this._compareWith = fn;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.checkboxes.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        () => {
            this._resetCheckboxs();
            this._initializeChecked();
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (this.checkboxes) {
            this._setCheckedByValue(value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @return {?}
     */
    focus() {
        if (!this.disabled) {
            this._onTouched();
        }
    }
    /**
     * @private
     * @return {?}
     */
    _initializeChecked() {
        Promise.resolve().then((/**
         * @return {?}
         */
        () => {
            this._setCheckedByValue(this.ngControl ? this.ngControl.value : this.value);
        }));
    }
    /**
     * @private
     * @return {?}
     */
    _resetCheckboxs() {
        /** @type {?} */
        const changedOrDestroyed = merge(this.checkboxes.changes, this._destroy);
        this.checkedChanges.pipe(takeUntil(changedOrDestroyed)).subscribe((/**
         * @return {?}
         */
        () => {
            this._setValues();
        }));
        merge(...this.checkboxes.map((/**
         * @param {?} item
         * @return {?}
         */
        item => item.change)))
            .pipe(takeUntil(changedOrDestroyed))
            .subscribe((/**
         * @return {?}
         */
        () => {
            this._changeDetectorRef.markForCheck();
        }));
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    _setCheckedByValue(value) {
        if (value) {
            this._value = value;
            this.checkboxes.forEach((/**
             * @param {?} item
             * @return {?}
             */
            item => item.checked = false));
            value.forEach((/**
             * @param {?} val
             * @return {?}
             */
            val => this._checkedValue(val)));
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    _checkedValue(value) {
        /** @type {?} */
        const correspondingItem = this.checkboxes.find((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            try {
                return item.value != null && this._compareWith(item.value, value);
            }
            catch (error) {
                if (isDevMode()) {
                    console.warn(error);
                }
                return false;
            }
        }));
        if (correspondingItem) {
            correspondingItem.checked = true;
        }
        return correspondingItem;
    }
    /**
     * @private
     * @return {?}
     */
    _setValues() {
        this._value = [];
        this.checkboxes.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (item.checked && !!item.value) {
                this._value.push(item.value);
            }
        }));
        this._onChange(this._value);
        this._onTouched();
        this._changeDetectorRef.markForCheck();
    }
}
NtCheckboxGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-checkbox-group',
                template: '<ng-content></ng-content>',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtCheckboxGroupComponent }
                ],
                host: {
                    'class': 'nt-checkbox-group'
                }
            },] },
];
/** @nocollapse */
NtCheckboxGroupComponent.ctorParameters = () => [
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] }
];
NtCheckboxGroupComponent.propDecorators = {
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    readonly: [{ type: Input }],
    checkboxes: [{ type: ContentChildren, args: [NtCheckboxComponent,] }],
    compareWith: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtCheckboxModule {
}
NtCheckboxModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtCheckboxComponent, NtCheckboxGroupComponent],
                declarations: [NtCheckboxComponent, NtCheckboxGroupComponent],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtCheckboxChange, NtCheckboxComponent, getNtCheckboxGroupNonFunctionValueError, NtCheckboxGroupComponent, NtCheckboxModule };
//# sourceMappingURL=checkbox.js.map
