import { ChangeDetectionStrategy, ChangeDetectorRef, Component, EventEmitter, Inject, Input, Optional, Output, ViewEncapsulation, ViewChild, InjectionToken, ElementRef, Self, NgModule } from '@angular/core';
import { DateAdapter, NT_DATE_FORMATS, BOTTOM_LEFT, fadeIn, fadeOut, NtOverlayComponent, TOP_LEFT, NtOverlayModule } from '@ng-tangram/components/core';
import { transition, trigger } from '@angular/animations';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { NgControl } from '@angular/forms';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
/**
 * @param {?} provider
 * @return {?}
 */
function createMissingDateImplError(provider) {
    return Error(`NtDatepickerComponent: No provider found for ${provider}. You must import one of the following ` +
        `modules at your application root: NtNativeDateModule or provide a ` +
        `custom implementation.`);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtDatePickerCell {
    /**
     * @param {?} value
     * @param {?} displayValue
     * @param {?} enabled
     */
    constructor(value, displayValue, enabled) {
        this.value = value;
        this.displayValue = displayValue;
        this.enabled = enabled;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DAYS_PER_WEEK = 7;
/** @type {?} */
const DAYS_MAX_ROWS = 6;
/**
 * @template D
 */
class NtDatePickerMonthComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} _dateAdapter
     * @param {?} _dateFormats
     */
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        /**
         * Emits when a new date is selected.
         */
        this.selectedChange = new EventEmitter();
        /**
         * Emits when any date is activated.
         */
        this.activeDateChange = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
        /** @type {?} */
        const firstDayOfWeek = this._dateAdapter.getFirstDayOfWeek();
        /** @type {?} */
        const narrowWeekdays = this._dateAdapter.getDayOfWeekNames('narrow');
        /** @type {?} */
        const longWeekdays = this._dateAdapter.getDayOfWeekNames('long');
        // Rotate the labels for days of the week based on the configured first day of the week.
        /** @type {?} */
        let weekdays = longWeekdays.map((/**
         * @param {?} long
         * @param {?} i
         * @return {?}
         */
        (long, i) => {
            return { long, narrow: narrowWeekdays[i] };
        }));
        this._weekdays = weekdays.slice(firstDayOfWeek).concat(weekdays.slice(0, firstDayOfWeek));
        this._activeDate = this._dateAdapter.today();
    }
    /**
     * The date to display in this month view (everything other than the month and year is ignored).
     * @return {?}
     */
    get activeDate() { return this._activeDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set activeDate(value) {
        /** @type {?} */
        const oldActiveDate = this._activeDate;
        /** @type {?} */
        const validDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (!this._hasSameMonthAndYear(oldActiveDate, this._activeDate)) {
            this._init();
        }
    }
    /**
     * The currently selected date.
     * @return {?}
     */
    get selected() { return this._selected; }
    /**
     * @param {?} value
     * @return {?}
     */
    set selected(value) {
        this._selected = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
        this._selectedDate = this._getDateInCurrentMonth(this._selected);
    }
    /**
     * The minimum selectable date.
     * @return {?}
     */
    get minDate() { return this._minDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minDate(value) {
        this._minDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * The maximum selectable date.
     * @return {?}
     */
    get maxDate() { return this._maxDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxDate(value) {
        this._maxDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._init();
    }
    /**
     * Handles when a new date is selected.
     * @param {?} cell
     * @return {?}
     */
    _dateSelected(cell) {
        if (cell.enabled && this._selectedDate !== cell.value) {
            /** @type {?} */
            const selectedYear = this._dateAdapter.getYear(this.activeDate);
            /** @type {?} */
            const selectedMonth = this._dateAdapter.getMonth(this.activeDate);
            /** @type {?} */
            const selectedDate = this._dateAdapter.createDate(selectedYear, selectedMonth, cell.value);
            this.selectedChange.emit(selectedDate);
        }
    }
    /**
     * @return {?}
     */
    _init() {
        this._selectedDate = this._getDateInCurrentMonth(this.selected);
        this._todayDate = this._getDateInCurrentMonth(this._dateAdapter.today());
        this._monthLabel = this._dateAdapter.getMonthNames('short')[this._dateAdapter.getMonth(this.activeDate)]
            .toLocaleUpperCase();
        /** @type {?} */
        const firstOfMonth = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), this._dateAdapter.getMonth(this.activeDate), 1);
        this._firstWeekOffset = (DAYS_PER_WEEK + this._dateAdapter.getDayOfWeek(firstOfMonth) -
            this._dateAdapter.getFirstDayOfWeek()) % DAYS_PER_WEEK;
        this._createWeekCells();
        this._changeDetectorRef.markForCheck();
    }
    // _todaySelected() {
    //   this.selectedChange.emit(this._dateAdapter.today());
    // }
    // _isTodaySelected() {
    //   return this._dateAdapter.sameDate(this.selected, this._dateAdapter.today());
    // }
    /**
     * Creates MatCalendarCells for the dates in this month.
     * @private
     * @return {?}
     */
    _createWeekCells() {
        /** @type {?} */
        const daysInMonth = this._dateAdapter.getNumDaysInMonth(this.activeDate);
        /** @type {?} */
        const dateNames = this._dateAdapter.getDateNames();
        this._weeks = [[]];
        for (let i = 0, cell = this._firstWeekOffset; i < daysInMonth; i++, cell++) {
            if (cell === DAYS_PER_WEEK) {
                this._weeks.push([]);
                cell = 0;
            }
            /** @type {?} */
            const date = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), this._dateAdapter.getMonth(this.activeDate), i + 1);
            /** @type {?} */
            const enabled = this._shouldEnableDate(date);
            this._weeks[this._weeks.length - 1].push(new NtDatePickerCell(i + 1, dateNames[i], enabled));
        }
        this._fillBeforeCells();
        this._fillAfterCells();
    }
    /**
     * @private
     * @return {?}
     */
    _getWeekOffset() {
        /** @type {?} */
        const firstDayForCurrentMonth = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), this._dateAdapter.getMonth(this.activeDate), 1);
        /** @type {?} */
        const firstDayForWeek = this._dateAdapter.getDayOfWeek(firstDayForCurrentMonth);
        return firstDayForWeek - this._dateAdapter.getFirstDayOfWeek() >= 0
            ? firstDayForWeek - this._dateAdapter.getFirstDayOfWeek()
            : firstDayForWeek - this._dateAdapter.getFirstDayOfWeek() + DAYS_PER_WEEK;
    }
    /**
     * 在填满模式下 补充首行列
     * @private
     * @return {?}
     */
    _fillBeforeCells() {
        /** @type {?} */
        const beforeMonth = this._dateAdapter.addCalendarMonths(this._dateAdapter.clone(this.activeDate), -1);
        /** @type {?} */
        const daysInBeforeMonth = this._dateAdapter.getNumDaysInMonth(beforeMonth);
        /** @type {?} */
        const dateNames = this._dateAdapter.getDateNames();
        /** @type {?} */
        const beforeWeeks = [];
        for (let i = daysInBeforeMonth - this._firstWeekOffset; i < daysInBeforeMonth; i++) {
            beforeWeeks.push(new NtDatePickerCell(i + 1, dateNames[i], false));
        }
        this._weeks[0] = beforeWeeks.concat(this._weeks[0]);
    }
    /**
     * 在填满模式下 补充尾部列
     * @private
     * @return {?}
     */
    _fillAfterCells() {
        /** @type {?} */
        const afterMonth = this._dateAdapter.addCalendarMonths(this._dateAdapter.clone(this.activeDate), 1);
        /** @type {?} */
        const lastRow = this._weeks[this._weeks.length - 1];
        /** @type {?} */
        const dateNames = this._dateAdapter.getDateNames();
        /** @type {?} */
        const afterWeeks = [];
        /** @type {?} */
        const afterRows = [];
        for (let i = 0; i < DAYS_PER_WEEK - lastRow.length; i++) {
            afterWeeks.push(new NtDatePickerCell(i + 1, dateNames[i], false));
        }
        lastRow.push(...afterWeeks);
        for (let i = 0; i < DAYS_PER_WEEK * (DAYS_MAX_ROWS - this._weeks.length); i++) {
            afterRows.push(new NtDatePickerCell(afterWeeks.length + i + 1, dateNames[afterWeeks.length + i], false));
        }
        this._weeks.push(...afterRows.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => afterRows.splice(0, DAYS_PER_WEEK))).filter((/**
         * @param {?} row
         * @return {?}
         */
        row => !!row)));
    }
    /**
     * Date filter for the month
     * @private
     * @param {?} date
     * @return {?}
     */
    _shouldEnableDate(date) {
        return !!date &&
            (!this.dateFilter || this.dateFilter(date)) &&
            (!this.minDate || this._dateAdapter.compareDate(date, this.minDate) >= 0) &&
            (!this.maxDate || this._dateAdapter.compareDate(date, this.maxDate) <= 0);
    }
    /**
     * Gets the date in this month that the given Date falls on.
     * Returns null if the given Date is in another month.
     * @private
     * @param {?} date
     * @return {?}
     */
    _getDateInCurrentMonth(date) {
        return date && this._hasSameMonthAndYear(date, this.activeDate) ?
            this._dateAdapter.getDate(date) : null;
    }
    /**
     * Checks whether the 2 dates are non-null and fall within the same month of the same year.
     * @private
     * @param {?} d1
     * @param {?} d2
     * @return {?}
     */
    _hasSameMonthAndYear(d1, d2) {
        return !!(d1 && d2 && this._dateAdapter.getMonth(d1) === this._dateAdapter.getMonth(d2) &&
            this._dateAdapter.getYear(d1) === this._dateAdapter.getYear(d2));
    }
    /**
     * @private
     * @param {?} obj The object to check.
     * @return {?} The given object if it is both a date instance and valid, otherwise null.
     */
    _getValidDateOrNull(obj) {
        return (this._dateAdapter.isDateInstance(obj) && this._dateAdapter.isValid(obj)) ? obj : null;
    }
}
NtDatePickerMonthComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-month',
                template: "<table> <thead> <tr> <th *ngFor=\"let week of _weekdays\">{{week.narrow}}</th> </tr> </thead> <tbody> <tr *ngFor=\"let _rows of _weeks\"> <td *ngFor=\"let week of _rows\" [class.disabled]=\"!week.enabled\" [class.selected]=\"week.value === _selectedDate && week.enabled\" [class.active]=\"_todayDate === week.value && week.enabled\"> <span (click)=\"_dateSelected(week)\">{{ week.displayValue }}</span> </td> </tr> </tbody> </table> <!-- <button class=\"button tiny expanded clear margin-bottom-0\" [disabled]=\"_isTodaySelected()\" (click)=\"_todaySelected()\">今日</button> --> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker-month'
                }
            },] },
];
/** @nocollapse */
NtDatePickerMonthComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] }
];
NtDatePickerMonthComponent.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    selectedChange: [{ type: Output }],
    activeDateChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const yearsPerPage = 12;
/** @type {?} */
const yearsPerRow = 3;
/**
 * @template D
 */
class NtDatePickerMultiYearComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} _dateAdapter
     */
    constructor(_changeDetectorRef, _dateAdapter) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        /**
         * Emits when a new year is selected.
         */
        this.selectedChange = new EventEmitter();
        /**
         * Emits the selected year. This doesn't imply a change on the selected date
         */
        this.yearSelected = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        this._activeDate = this._dateAdapter.today();
    }
    /**
     * @return {?}
     */
    get activeDate() { return this._activeDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set activeDate(value) {
        /** @type {?} */
        let oldActiveDate = this._activeDate;
        /** @type {?} */
        const validDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (Math.floor(this._dateAdapter.getYear(oldActiveDate) / yearsPerPage) !==
            Math.floor(this._dateAdapter.getYear(this._activeDate) / yearsPerPage)) {
            this._init();
        }
    }
    /**
     * The currently selected date.
     * @return {?}
     */
    get selected() { return this._selected; }
    /**
     * @param {?} value
     * @return {?}
     */
    set selected(value) {
        this._selected = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
        this._selectedYear = this._selected && this._dateAdapter.getYear(this._selected);
    }
    /**
     * The minimum selectable date.
     * @return {?}
     */
    get minDate() { return this._minDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minDate(value) {
        this._minDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * The maximum selectable date.
     * @return {?}
     */
    get maxDate() { return this._maxDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxDate(value) {
        this._maxDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._init();
    }
    /**
     * @return {?}
     */
    _init() {
        this._todayYear = this._dateAdapter.getYear(this._dateAdapter.today());
        /** @type {?} */
        let activeYear = this._dateAdapter.getYear(this._activeDate);
        /** @type {?} */
        let activeOffset = activeYear % yearsPerPage;
        this._years = [];
        for (let i = 0, row = []; i < yearsPerPage; i++) {
            row.push(activeYear - activeOffset + i);
            if (row.length === yearsPerRow) {
                this._years.push(row.map((/**
                 * @param {?} year
                 * @return {?}
                 */
                year => this._createCellForYear(year))));
                row = [];
            }
        }
        this._changeDetectorRef.markForCheck();
    }
    /**
     * Handles when a new year is selected.
     * @param {?} year
     * @return {?}
     */
    _yearSelected(year) {
        this.yearSelected.emit(this._dateAdapter.createDate(year, 0, 1));
        /** @type {?} */
        let month = this._dateAdapter.getMonth(this.activeDate);
        /** @type {?} */
        let daysInMonth = this._dateAdapter.getNumDaysInMonth(this._dateAdapter.createDate(year, month, 1));
        this.selectedChange.emit(this._dateAdapter.createDate(year, month, Math.min(this._dateAdapter.getDate(this.activeDate), daysInMonth)));
    }
    /**
     * Creates an MatCalendarCell for the given year.
     * @private
     * @param {?} year
     * @return {?}
     */
    _createCellForYear(year) {
        /** @type {?} */
        let yearName = this._dateAdapter.getYearName(this._dateAdapter.createDate(year, 0, 1));
        return new NtDatePickerCell(year, yearName, this._shouldEnableYear(year));
    }
    /**
     * Whether the given year is enabled.
     * @private
     * @param {?} year
     * @return {?}
     */
    _shouldEnableYear(year) {
        // disable if the year is greater than maxDate lower than minDate
        if (year === undefined || year === null ||
            (this.maxDate && year > this._dateAdapter.getYear(this.maxDate)) ||
            (this.minDate && year < this._dateAdapter.getYear(this.minDate))) {
            return false;
        }
        // enable if it reaches here and there's no filter defined
        if (!this.dateFilter) {
            return true;
        }
        /** @type {?} */
        const firstOfYear = this._dateAdapter.createDate(year, 0, 1);
        // If any date in the year is enabled count the year as enabled.
        for (let date = firstOfYear; this._dateAdapter.getYear(date) === year; date = this._dateAdapter.addCalendarDays(date, 1)) {
            if (this.dateFilter(date)) {
                return true;
            }
        }
        return false;
    }
    /**
     * @private
     * @param {?} obj The object to check.
     * @return {?} The given object if it is both a date instance and valid, otherwise null.
     */
    _getValidDateOrNull(obj) {
        return (this._dateAdapter.isDateInstance(obj) && this._dateAdapter.isValid(obj)) ? obj : null;
    }
}
NtDatePickerMultiYearComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-multi-year',
                template: "<table> <tbody> <tr *ngFor=\"let _rows of _years\"> <td *ngFor=\"let year of _rows\" [class.disabled]=\"!year.enabled\" [class.selected]=\"year.value === _selectedYear && year.enabled\" (click)=\"_yearSelected(year.value)\"> <span *ngIf=\"year.enabled\">{{ year.displayValue }}</span> <span *ngIf=\"!year.enabled\">{{ year.displayValue }}</span> </td> </tr> </tbody> </table> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker-multi-year'
                }
            },] },
];
/** @nocollapse */
NtDatePickerMultiYearComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] }
];
NtDatePickerMultiYearComponent.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    selectedChange: [{ type: Output }],
    yearSelected: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template D
 */
class NtDatePickerYearComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} _dateAdapter
     * @param {?} _dateFormats
     */
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        /**
         * Emits when a new month is selected.
         */
        this.selectedChange = new EventEmitter();
        /**
         * Emits the selected month. This doesn't imply a change on the selected date
         */
        this.monthSelected = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
        this._activeDate = this._dateAdapter.today();
    }
    /**
     * @return {?}
     */
    get activeDate() { return this._activeDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set activeDate(value) {
        /** @type {?} */
        let oldActiveDate = this._activeDate;
        /** @type {?} */
        const validDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (this._dateAdapter.getYear(oldActiveDate) !== this._dateAdapter.getYear(this._activeDate)) {
            this._init();
        }
    }
    /**
     * @return {?}
     */
    get activeYear() { return this._activeDate ? this._dateAdapter.getYear(this._activeDate) : null; }
    /**
     * The currently selected date.
     * @return {?}
     */
    get selected() { return this._selected; }
    /**
     * @param {?} value
     * @return {?}
     */
    set selected(value) {
        this._selected = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
        this._selectedMonth = this._getMonthInCurrentYear(this._selected);
    }
    /**
     * The minimum selectable date.
     * @return {?}
     */
    get minDate() { return this._minDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minDate(value) {
        this._minDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * The maximum selectable date.
     * @return {?}
     */
    get maxDate() { return this._maxDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxDate(value) {
        this._maxDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._init();
    }
    /**
     * @return {?}
     */
    _init() {
        this._selectedMonth = this._getMonthInCurrentYear(this.selected);
        this._todayMonth = this._getMonthInCurrentYear(this._dateAdapter.today());
        /** @type {?} */
        const monthNames = this._dateAdapter.getMonthNames('short');
        this._months = [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9, 10, 11]].map((/**
         * @param {?} row
         * @return {?}
         */
        row => row.map((/**
         * @param {?} month
         * @return {?}
         */
        month => this._createCellForMonth(month, monthNames[month])))));
        this._changeDetectorRef.markForCheck();
    }
    /**
     * Handles when a new month is selected.
     * @param {?} month
     * @return {?}
     */
    _monthSelected(month) {
        /** @type {?} */
        const normalizedDate = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), month, 1);
        this.monthSelected.emit(normalizedDate);
        /** @type {?} */
        const daysInMonth = this._dateAdapter.getNumDaysInMonth(normalizedDate);
        this.selectedChange.emit(this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), month, Math.min(this._dateAdapter.getDate(this.activeDate), daysInMonth)));
    }
    /**
     * Creates an MatCalendarCell for the given month.
     * @private
     * @param {?} month
     * @param {?} monthName
     * @return {?}
     */
    _createCellForMonth(month, monthName) {
        return new NtDatePickerCell(month, monthName.toLocaleUpperCase(), this._shouldEnableMonth(month));
    }
    /**
     * Whether the given month is enabled.
     * @private
     * @param {?} month
     * @return {?}
     */
    _shouldEnableMonth(month) {
        /** @type {?} */
        const activeYear = this._dateAdapter.getYear(this.activeDate);
        if (month === undefined || month === null ||
            this._isYearAndMonthAfterMaxDate(activeYear, month) ||
            this._isYearAndMonthBeforeMinDate(activeYear, month)) {
            return false;
        }
        if (!this.dateFilter) {
            return true;
        }
        /** @type {?} */
        const firstOfMonth = this._dateAdapter.createDate(activeYear, month, 1);
        // If any date in the month is enabled count the month as enabled.
        for (let date = firstOfMonth; this._dateAdapter.getMonth(date) === month; date = this._dateAdapter.addCalendarDays(date, 1)) {
            if (this.dateFilter(date)) {
                return true;
            }
        }
        return false;
    }
    /**
     * Tests whether the combination month/year is after this.maxDate, considering
     * just the month and year of this.maxDate
     * @private
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    _isYearAndMonthAfterMaxDate(year, month) {
        if (this.maxDate) {
            /** @type {?} */
            const maxYear = this._dateAdapter.getYear(this.maxDate);
            /** @type {?} */
            const maxMonth = this._dateAdapter.getMonth(this.maxDate);
            return year > maxYear || (year === maxYear && month > maxMonth);
        }
        return false;
    }
    /**
     * Tests whether the combination month/year is before this.minDate, considering
     * just the month and year of this.minDate
     * @private
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    _isYearAndMonthBeforeMinDate(year, month) {
        if (this.minDate) {
            /** @type {?} */
            const minYear = this._dateAdapter.getYear(this.minDate);
            /** @type {?} */
            const minMonth = this._dateAdapter.getMonth(this.minDate);
            return year < minYear || (year === minYear && month < minMonth);
        }
        return false;
    }
    /**
     * Gets the month in this year that the given Date falls on.
     * Returns null if the given Date is in another year.
     * @private
     * @param {?} date
     * @return {?}
     */
    _getMonthInCurrentYear(date) {
        return date && this._dateAdapter.getYear(date) === this._dateAdapter.getYear(this.activeDate) ?
            this._dateAdapter.getMonth(date) : null;
    }
    /**
     * @private
     * @param {?} obj The object to check.
     * @return {?} The given object if it is both a date instance and valid, otherwise null.
     */
    _getValidDateOrNull(obj) {
        return (this._dateAdapter.isDateInstance(obj) && this._dateAdapter.isValid(obj)) ? obj : null;
    }
}
NtDatePickerYearComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-year',
                template: "<table> <tbody> <tr *ngFor=\"let _rows of _months\"> <td *ngFor=\"let month of _rows\" [class.disabled]=\"!month.enabled\" [class.selected]=\"month.value === _selectedMonth && month.enabled\" (click)=\"_monthSelected(month.value)\"> <span *ngIf=\"month.enabled\">{{ month.displayValue }}</span> <span *ngIf=\"!month.enabled\">{{ month.displayValue }}</span> </td> </tr> </tbody> </table> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker-year'
                }
            },] },
];
/** @nocollapse */
NtDatePickerYearComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] }
];
NtDatePickerYearComponent.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    selectedChange: [{ type: Output }],
    monthSelected: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template D
 */
class NtDatePickerCalendarComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} _dateAdapter
     * @param {?} _dateFormats
     */
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this._viewType = 'month';
        this._currentView = 'month';
        /**
         * Emits when the currently selected date changes.
         */
        this.selectedChange = new EventEmitter();
        /**
         * Emits the year chosen in multiyear view.
         * This doesn't imply a change on the selected date.
         */
        this.yearSelected = new EventEmitter();
        /**
         * Emits the month chosen in year view.
         * This doesn't imply a change on the selected date.
         */
        this.monthSelected = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
    }
    /**
     * @return {?}
     */
    get startAt() { return this._startAt; }
    /**
     * @param {?} value
     * @return {?}
     */
    set startAt(value) { this._startAt = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get selected() { return this._selected; }
    /**
     * @param {?} value
     * @return {?}
     */
    set selected(value) { this._selected = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get minDate() { return this._minDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minDate(value) { this._minDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get maxDate() { return this._maxDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxDate(value) { this._maxDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get _activeDate() { return this._clampedActiveDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set _activeDate(value) { this._clampedActiveDate = this._dateAdapter.clampDate(value, this.minDate, this.maxDate); }
    /**
     * @return {?}
     */
    get _monthLabel() {
        return this._dateAdapter.format(this._activeDate, this._dateFormats.display.monthLabel).toLocaleUpperCase();
    }
    /**
     * @return {?}
     */
    get _yearLabel() { return this._dateAdapter.getYearName(this._activeDate); }
    /**
     * @return {?}
     */
    get _multiYearLabel() {
        /** @type {?} */
        const activeYear = this._dateAdapter.getYear(this._activeDate);
        /** @type {?} */
        const firstYearInView = this._dateAdapter.getYearName(this._dateAdapter.createDate(activeYear - activeYear % yearsPerPage, 0, 1), 'en-US');
        /** @type {?} */
        const lastYearInView = this._dateAdapter.getYearName(this._dateAdapter.createDate(activeYear + yearsPerPage - 1 - activeYear % yearsPerPage, 0, 1), 'en-US');
        return `${firstYearInView} \u2013 ${lastYearInView}`;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._init();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const change = changes.minDate || changes.maxDate || changes.dateFilter;
        if (change && !change.firstChange) {
            /** @type {?} */
            const view = this.monthView || this.yearView || this.multiYearView;
            if (view) {
                view._init();
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // throw new Error("Method not implemented.");
    }
    /**
     * @return {?}
     */
    prevMonth() {
        this._activeDate = this._dateAdapter.addCalendarMonths(this._activeDate, -1);
    }
    /**
     * @return {?}
     */
    nextMonth() {
        this._activeDate = this._dateAdapter.addCalendarMonths(this._activeDate, 1);
    }
    /**
     * @return {?}
     */
    prevYear() {
        this._activeDate = this._dateAdapter.addCalendarYears(this._activeDate, -1);
    }
    /**
     * @return {?}
     */
    nextYear() {
        this._activeDate = this._dateAdapter.addCalendarYears(this._activeDate, 1);
    }
    /**
     * @return {?}
     */
    prevYearArray() {
        this._activeDate = this._dateAdapter.addCalendarYears(this._activeDate, -yearsPerPage);
    }
    /**
     * @return {?}
     */
    nextYearArray() {
        this._activeDate = this._dateAdapter.addCalendarYears(this._activeDate, yearsPerPage);
    }
    /**
     * @return {?}
     */
    _init() {
        this._activeDate = this.startAt || this._dateAdapter.today();
        this._currentView = this._viewType;
        this._changeDetectorRef.markForCheck();
    }
    /**
     * Handles date selection in the month view.
     * @param {?} date
     * @return {?}
     */
    _dateSelected(date) {
        if (!this._dateAdapter.sameDate(date, this.selected)) {
            this.selectedChange.emit(date);
        }
    }
    /**
     * Handles year selection in the multiyear view.
     * @param {?} normalizedYear
     * @return {?}
     */
    _yearSelectedInMultiYearView(normalizedYear) {
        this.yearSelected.emit(normalizedYear);
    }
    /**
     * Handles month selection in the year view.
     * @param {?} normalizedMonth
     * @return {?}
     */
    _monthSelectedInYearView(normalizedMonth) {
        this.monthSelected.emit(normalizedMonth);
    }
    /**
     * Handles year/month selection in the multi-year/year views.
     * @param {?} date
     * @param {?} view
     * @return {?}
     */
    _goToDateInView(date, view) {
        this._activeDate = date;
        this._currentView = view;
    }
    /**
     * @private
     * @param {?} obj The object to check.
     * @return {?} The given object if it is both a date instance and valid, otherwise null.
     */
    _getValidDateOrNull(obj) {
        return (this._dateAdapter.isDateInstance(obj) && this._dateAdapter.isValid(obj)) ? obj : null;
    }
}
NtDatePickerCalendarComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-calendar',
                template: "<div class=\"nt-datepicker-pane\" [ngSwitch]=\"_currentView\"> <table> <thead> <tr *ngSwitchCase=\"'month'\"><ng-container *ngTemplateOutlet=\"monthControl\"></ng-container></tr> <tr *ngSwitchCase=\"'year'\"><ng-container *ngTemplateOutlet=\"yearControl\"></ng-container></tr> <tr *ngSwitchCase=\"'multi-year'\"><ng-container *ngTemplateOutlet=\"multiYearControl\"></ng-container></tr> </thead> </table> <nt-datepicker-month *ngSwitchCase=\"'month'\" [(activeDate)]=\"_activeDate\" [selected]=\"selected\" [dateFilter]=\"dateFilter\" [maxDate]=\"maxDate\" [minDate]=\"minDate\" (selectedChange)=\"_dateSelected($event)\"> </nt-datepicker-month> <nt-datepicker-year *ngSwitchCase=\"'year'\" [activeDate]=\"_activeDate\" [selected]=\"selected\" [dateFilter]=\"dateFilter\" [maxDate]=\"maxDate\" [minDate]=\"minDate\" (monthSelected)=\"_monthSelectedInYearView($event)\" (selectedChange)=\"_goToDateInView($event, 'month')\"> </nt-datepicker-year> <nt-datepicker-multi-year *ngSwitchCase=\"'multi-year'\" [activeDate]=\"_activeDate\" [selected]=\"selected\" [dateFilter]=\"dateFilter\" [maxDate]=\"maxDate\" [minDate]=\"minDate\" (yearSelected)=\"_yearSelectedInMultiYearView($event)\" (selectedChange)=\"_goToDateInView($event, 'year')\"> </nt-datepicker-multi-year> </div> <ng-template #monthControl> <th (click)=\"prevYear()\" [class.disabled]=\"\"><span>&laquo;</span></th> <th (click)=\"prevMonth()\" [class.disabled]=\"\"><span>&lsaquo;</span></th> <th colspan=\"3\"> <span (click)=\" _currentView = 'multi-year' \">{{ _yearLabel }}</span> <span (click)=\" _currentView = 'year' \">{{ _monthLabel }}</span> </th> <th (click)=\"nextMonth()\" [class.disabled]=\"\"><span>&rsaquo;</span></th> <th (click)=\"nextYear()\" [class.disabled]=\"\"><span>&raquo;</span></th> </ng-template> <ng-template #yearControl> <th (click)=\"prevYear()\" [class.disabled]=\"\"><span>&laquo;</span></th> <th>&nbsp;</th> <th colspan=\"3\"> <span (click)=\" _currentView = 'multi-year' \">{{ _yearLabel }}</span> </th> <th>&nbsp;</th> <th (click)=\"nextYear()\" [class.disabled]=\"\"><span>&raquo;</span></th> </ng-template> <ng-template #multiYearControl> <th (click)=\"prevYearArray()\"><span>&laquo;</span></th> <th>&nbsp;</th> <th colspan=\"3\">{{_multiYearLabel}}</th> <th>&nbsp;</th> <th (click)=\"nextYearArray()\"><span>&raquo;</span></th> </ng-template> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] },
];
/** @nocollapse */
NtDatePickerCalendarComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] }
];
NtDatePickerCalendarComponent.propDecorators = {
    startAt: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    selectedChange: [{ type: Output }],
    yearSelected: [{ type: Output }],
    monthSelected: [{ type: Output }],
    monthView: [{ type: ViewChild, args: [NtDatePickerMonthComponent, { static: false },] }],
    yearView: [{ type: ViewChild, args: [NtDatePickerYearComponent, { static: false },] }],
    multiYearView: [{ type: ViewChild, args: [NtDatePickerMultiYearComponent, { static: false },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_DATEPICKER_ICONS = new InjectionToken('nt-datepicker-icons');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template D
 */
class NtDatePickerComponent extends NtFormFieldControl {
    /**
     * @param {?} _elementRef
     * @param {?} _dateAdapter
     * @param {?} _dateFormats
     * @param {?} ngControl
     * @param {?} icons
     */
    constructor(_elementRef, _dateAdapter, _dateFormats, ngControl, icons) {
        super();
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this.ngControl = ngControl;
        this.icons = icons;
        this._disabled = false;
        this._lastValueValid = false;
        this._readonly = false;
        this._required = false;
        this._focused = false;
        this._positionPairs = [BOTTOM_LEFT, TOP_LEFT];
        this.placeholder = '';
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        /**
         * Emits when the value changes (either due to user input or programmatic change).
         */
        this._valueChange = new EventEmitter();
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        this.origin = new CdkOverlayOrigin(_elementRef);
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get empty() { return !this.value; }
    /**
     * @return {?}
     */
    get focused() { return this._focused; }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        value = this._dateAdapter.deserialize(value);
        this._lastValueValid = !value || this._dateAdapter.isValid(value);
        value = this._getValidDateOrNull(value);
        /** @type {?} */
        const oldDate = this.value;
        this._value = value;
        this.inputElement.nativeElement.value = value ? this._dateAdapter.format(value, this._dateFormats.display.dateInput) : '';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @return {?}
     */
    get required() { return this._required; }
    /**
     * @param {?} value
     * @return {?}
     */
    set required(value) { this._required = coerceBooleanProperty(value); }
    /**
     * @param {?} value
     * @return {?}
     */
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get readonly() { return this._readonly; }
    /**
     * @return {?}
     */
    get startAt() { return this._startAt || this.value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set startAt(value) { this._startAt = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get minDate() { return this._minDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set minDate(value) { this._minDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @return {?}
     */
    get maxDate() { return this._maxDate; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxDate(value) { this._maxDate = this._getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.value = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @return {?}
     */
    _onInputFocus() {
        if (!this.disabled) {
            this.overlay.show();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeOpen(event) {
        this._focused = true;
        this.calendar._init();
        this.beforeOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeClosed(event) {
        this._focused = false;
        typeof this._onTouched === 'function' && this._onTouched();
        this.inputElement.nativeElement.blur();
        this.beforeClosed.next(event);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    _positionChange(change) {
        this.positionChange.next(change);
    }
    /**
     * @return {?}
     */
    focus() {
        if (!this.disabled) {
            this.inputElement.nativeElement.focus();
        }
    }
    /**
     * @param {?} date
     * @return {?}
     */
    select(date) {
        this.value = date;
        this.overlay.hide();
        this._onChange(date);
    }
    /**
     * @return {?}
     */
    clear() {
        if (this.value !== null && !this.disabled) {
            this.value = null;
            this._onChange(this.value);
        }
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this._disabled = isDisabled;
    }
    /**
     * @private
     * @param {?} obj The object to check.
     * @return {?} The given object if it is both a date instance and valid, otherwise null.
     */
    _getValidDateOrNull(obj) {
        return (this._dateAdapter.isDateInstance(obj) && this._dateAdapter.isValid(obj)) ? obj : null;
    }
}
NtDatePickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker',
                template: "<input #inputElement type=\"text\" (focus)=\"_onInputFocus()\" readonly [placeholder]=\"placeholder\" [disabled]=\"disabled\"> <span class=\"nt-datepicker-caret {{ icons.caret }}\" [@fade] *ngIf=\"!value || disabled\"></span> <span class=\"nt-datepicker-clear {{ icons.clear }}\" (click)=\"clear()\" [@fade] *ngIf=\"value && !disabled\"></span> <nt-overlay [origin]=\"origin\" trigger=\"click\" [positionPairs]=\"_positionPairs\" arrow (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\"> <nt-datepicker-calendar [selected]=\"value\" [dateFilter]=\"dateFilter\" [startAt]=\"startAt\" [maxDate]=\"maxDate\" [minDate]=\"minDate\" (selectedChange)=\"select($event)\"> </nt-datepicker-calendar> </nt-overlay> ",
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'nt-datepicker',
                    '[class.focus]': 'overlay.isOpen'
                },
                animations: [
                    trigger('fade', [
                        transition('* => void', fadeOut(.15)),
                        transition('void => *', fadeIn(.15))
                    ])
                ],
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtDatePickerComponent }
                ]
            },] },
];
/** @nocollapse */
NtDatePickerComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: DateAdapter },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATEPICKER_ICONS,] }] }
];
NtDatePickerComponent.propDecorators = {
    placeholder: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    readonly: [{ type: Input }],
    startAt: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    inputElement: [{ type: ViewChild, args: ['inputElement', { static: true },] }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }],
    calendar: [{ type: ViewChild, args: [NtDatePickerCalendarComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_DATEPICKER_ICONS = {
    caret: 'far fa-calendar-alt',
    clear: 'fa fa-times'
};
const ɵ0 = DEFAULT_DATEPICKER_ICONS;
class NtDatePickerModule {
    /**
     * @param {?=} icons
     * @return {?}
     */
    static forRoot(icons = DEFAULT_DATEPICKER_ICONS) {
        return {
            ngModule: NtDatePickerModule,
            providers: [
                { provide: NT_DATEPICKER_ICONS, useValue: icons }
            ]
        };
    }
}
NtDatePickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtOverlayModule],
                exports: [NtDatePickerComponent],
                declarations: [
                    NtDatePickerComponent,
                    NtDatePickerCalendarComponent,
                    NtDatePickerMonthComponent,
                    NtDatePickerYearComponent,
                    NtDatePickerMultiYearComponent
                ],
                providers: [
                    { provide: NT_DATEPICKER_ICONS, useValue: ɵ0 }
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtDatePickerModule, NtDatePickerComponent, createMissingDateImplError, NtDatePickerCell, NT_DATEPICKER_ICONS, NtDatePickerCalendarComponent, NtDatePickerMonthComponent, yearsPerPage, yearsPerRow, NtDatePickerMultiYearComponent, NtDatePickerYearComponent };
//# sourceMappingURL=datepicker.js.map
