import { InjectionToken, Pipe, Component, EventEmitter, Inject, Input, Optional, Output, Self, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { transition, trigger } from '@angular/animations';
import { coerceArray, coerceBooleanProperty, coerceNumberProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { fadeOut, NT_UPLOAD_HANDLER, NtUploadFile, NtUploadStatus } from '@ng-tangram/components/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';
import { NtProgressModule } from '@ng-tangram/components/progress';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_FILE_ICONS = {
    default: 'far fa-file-alt',
    archive: 'far fa-file-archive',
    code: 'far fa-file-code',
    image: 'far fa-file-image',
    audio: 'far fa-file-audio',
    video: 'far fa-file-video',
    pdf: 'far fa-file-pdf',
    word: 'far fa-file-word',
    excel: 'far fa-file-excel',
    ppt: 'far fa-file-powerpoint',
};
/** @type {?} */
const NT_FILE_ICONS = new InjectionToken('nt-file-icons');
/** @type {?} */
const NT_FILE_EXTENSIONS = {
    'rar': 'archive',
    'zip': 'archive',
    'cab': 'archive',
    'iso': 'archive',
    'jar': 'archive',
    '7z': 'archive',
    'tar': 'archive',
    'gz': 'archive',
    'bz2': 'archive',
    'z': 'archive',
    'jpg': 'image',
    'bmp': 'image',
    'eps': 'image',
    'gif': 'image',
    'mif': 'image',
    'miff': 'image',
    'png': 'image',
    'tif': 'image',
    'tiff': 'image',
    'svg': 'image',
    'wmf': 'image',
    'jpe': 'image',
    'jpeg': 'image',
    'dib': 'image',
    'ico': 'image',
    'tga': 'image',
    'cut': 'image',
    'pic': 'image',
    'wma': 'audio',
    'ra': 'audio',
    'ogg': 'audio',
    'mpc': 'audio',
    'm4a': 'audio',
    'aac': 'audio',
    'mpa': 'audio',
    'mp2': 'audio',
    'm1a': 'audio',
    'm2a': 'audio',
    'mp3': 'audio',
    'mid': 'audio',
    'midi': 'audio',
    'rmi': 'audio',
    'mka': 'audio',
    'ac3': 'audio',
    'dts': 'audio',
    'cda': 'audio',
    'au': 'audio',
    'snd': 'audio',
    'aif': 'audio',
    'aifc': 'audio',
    'aiff': 'audio',
    'mp4': 'video',
    'avi': 'video',
    'wav': 'video',
    'wmv': 'video',
    'rmvb': 'video',
    'rm': 'video',
    'flv': 'video',
    'mkv': 'video',
    'mpg': 'video',
    'mpeg': 'video',
    'mov': 'video',
    '3gp': 'video',
    'doc': 'word',
    'docx': 'word',
    'xls': 'excel',
    'xlsx': 'excel',
    'ppt': 'ppt',
    'pptx': 'ppt'
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const FILE_SIZE_UNITS = ['bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];
class NtFileSizePipe {
    /**
     * @param {?=} bytes
     * @param {?=} precision
     * @return {?}
     */
    transform(bytes = 0, precision = 2) {
        if (isNaN(parseFloat(String(bytes))) || !isFinite(bytes)) {
            return '?';
        }
        /** @type {?} */
        let unit = 0;
        while (bytes >= 1024) {
            bytes /= 1024;
            unit++;
        }
        return bytes.toFixed(+precision) + ' ' + FILE_SIZE_UNITS[unit];
    }
}
NtFileSizePipe.decorators = [
    { type: Pipe, args: [{ name: 'ntFileSize' },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFileAcceptError {
    /**
     * @param {?} file
     * @param {?} fileAccept
     * @param {?=} allowAccepts
     */
    constructor(file, fileAccept, allowAccepts) {
        this.file = file;
        this.fileAccept = fileAccept;
        this.allowAccepts = allowAccepts;
    }
}
class NtFileSizeError {
    /**
     * @param {?} file
     * @param {?} maxSize
     * @param {?=} maxSizeString
     */
    constructor(file, maxSize, maxSizeString) {
        this.file = file;
        this.maxSize = maxSize;
        this.maxSizeString = maxSizeString;
    }
}
class NtFileUploadError {
    /**
     * @param {?} status
     * @param {?} statusText
     */
    constructor(status, statusText) {
        this.status = status;
        this.statusText = statusText;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
let uniqueId = 0;
class NtFile extends NtUploadFile {
    constructor() {
        super(...arguments);
        this.id = `nt-file-${uniqueId++}`;
    }
}
class NtFileComponent extends NtFormFieldControl {
    /**
     * @param {?} ngControl
     * @param {?} _uploader
     * @param {?} icons
     */
    constructor(ngControl, _uploader, icons) {
        super();
        this.ngControl = ngControl;
        this._uploader = _uploader;
        this.icons = icons;
        this._destroy = new Subject();
        this.files = [];
        this._value = [];
        this._disabled = false;
        this._required = false;
        this._accept = ['*'];
        this._maxSize = 5;
        this._maxFiles = 1;
        this._autoupload = true;
        this.url = '';
        this.name = '';
        this.error = new EventEmitter();
        this.remove = new EventEmitter();
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @return {?}
     */
    get required() { return this._required; }
    /**
     * @param {?} value
     * @return {?}
     */
    set required(value) { this._required = coerceBooleanProperty(value); }
    /**
     * @param {?} value
     * @return {?}
     */
    set accept(value) {
        if (typeof value === 'string') {
            this._accept = value.replace(' ', '').split(',');
        }
        else {
            this._accept = coerceArray(value);
        }
    }
    /**
     * @return {?}
     */
    get accept() { return this._accept; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxSize(value) { this._maxSize = coerceNumberProperty(value, 5); }
    /**
     * @return {?}
     */
    get maxSize() { return this._maxSize; }
    /**
     * @param {?} value
     * @return {?}
     */
    set maxFiles(value) { this._maxFiles = coerceNumberProperty(value, 1); }
    /**
     * @return {?}
     */
    get maxFiles() { return this._maxFiles; }
    /**
     * @param {?} value
     * @return {?}
     */
    set autoupload(value) { this._autoupload = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get autoupload() { return this._autoupload; }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        this.error.complete();
        this.remove.complete();
    }
    /**
     * @return {?}
     */
    onTriggerClick() {
        if (this.files.length < this.maxFiles) {
            this.fileElement.nativeElement.click();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value) {
            this._value.length = 0;
            this._value.push(...value);
            this.files.length = 0;
            this.files.push(...this._value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @return {?}
     */
    _fileChanged() {
        /** @type {?} */
        const file = this.fileElement.nativeElement.files[0];
        if (file && this.files.length < this.maxFiles) {
            if (!this._fileSizeValid(file)) {
                this.error.next(new NtFileSizeError(file, this.maxSize * 1024 * 1024, `${this.maxSize}MB`));
                return;
            }
            if (!this._fileTypeValid(file)) {
                this.error.next(new NtFileAcceptError(file, file.type));
                return;
            }
            /** @type {?} */
            let ntFile = new NtFile(file.name, file.size, file.type);
            this.files.push(ntFile);
            /** @type {?} */
            const handlers = {
                begin: (/**
                 * @return {?}
                 */
                () => this._onUploadBegin(ntFile)),
                progress: (/**
                 * @param {?} percent
                 * @return {?}
                 */
                percent => this._onUploadProgress(percent, ntFile)),
                done: (/**
                 * @return {?}
                 */
                () => this._onUploadDone(ntFile))
            };
            if (this.autoupload) {
                ntFile.uploader = this._uploader
                    .upload(this.url, file, this.name, handlers)
                    .pipe(takeUntil(this._destroy))
                    .subscribe((/**
                 * @param {?} result
                 * @return {?}
                 */
                result => {
                    if ((ntFile.status = result.status) === NtUploadStatus.SUCCESS) {
                        ntFile.data = result.data;
                        this._value.push(ntFile);
                        this._onChange(this._value);
                    }
                }), (/**
                 * @param {?} error
                 * @return {?}
                 */
                error => {
                    ntFile.status = NtUploadStatus.ERROR;
                    ntFile.error = error.statusText;
                    ntFile.progress = 100;
                    this.error.next(new NtFileUploadError(error.status, error.statusText));
                }));
            }
        }
        this._onTouched && this._onTouched();
        this.fileElement.nativeElement.value = '';
    }
    /**
     * @param {?} file
     * @return {?}
     */
    removeFile(file) {
        if (this.disabled) {
            return;
        }
        file.uploader && file.uploader.unsubscribe();
        /** @type {?} */
        const vindex = this._value.indexOf(file);
        vindex > -1 && this._value.splice(vindex, 1);
        /** @type {?} */
        const findex = this.files.indexOf(file);
        findex > -1 && this.files.splice(findex, 1);
        this.remove.next(file);
        this._onChange(this._value);
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this._disabled = isDisabled;
    }
    /**
     * @param {?} file
     * @return {?}
     */
    getFileIcon(file) {
        /** @type {?} */
        const extension = file.name.substring(file.name.lastIndexOf('.') + 1, file.name.length);
        /** @type {?} */
        const icon = NT_FILE_EXTENSIONS[extension] || 'default';
        return this.icons[icon];
    }
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    _fileTypeValid(file) {
        if (file) {
            return this.accept.indexOf(file.type) > -1 || this.accept.indexOf('*') > -1;
        }
        return false;
    }
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    _fileSizeValid(file) {
        if (file) {
            return file.size <= this.maxSize * 1024 * 1024;
        }
        return false;
    }
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    _onUploadBegin(file) {
        file.status = NtUploadStatus.SENDING;
    }
    /**
     * @private
     * @param {?} percent
     * @param {?} file
     * @return {?}
     */
    _onUploadProgress(percent, file) {
        file.progress = percent;
    }
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    _onUploadDone(file) {
        file.status = NtUploadStatus.SUCCESS;
    }
}
NtFileComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-file',
                template: "<button class=\"button hollow nt-file-trigger\" type=\"button\" (click)=\"onTriggerClick()\" [disabled]=\"files.length >= maxFiles || disabled\"> <ng-content></ng-content> </button> <input #fileElement type=\"file\" [accept]=\"accept\" (change)=\"_fileChanged()\" hidden> <div class=\"nt-file-list\"> <div class=\"nt-file-item\" *ngFor=\"let file of files\" [@fadeOut]> <div class=\"nt-file-info\" [class.success]=\"file.status === 2\" [class.error]=\"file.status === 3\" [class.remote]=\"file.status === 4\"> <span class=\"nt-file-symbol {{ getFileIcon(file) }}\"></span>&nbsp; <a *ngIf=\"file.link\" [href]=\"file.link\" target=\"_blank\">{{file.name}}</a> <ng-container *ngIf=\"!file.link\">{{file.name}}</ng-container> &nbsp;{{ file.size | ntFileSize }} &nbsp;<span class=\"nt-file-error\" *ngIf=\"file.error\">Error&nbsp;-&nbsp;{{file.error}}</span> <div class=\"nt-close-button\" *ngIf=\"!disabled\" (click)=\"removeFile(file)\"><span aria-hidden=\"true\">&times;</span></div> </div> <nt-progress *ngIf=\"file.status === 1\" [@fadeOut] [class.success]=\"file.status === 2\" [class.alert]=\"file.status === 3\" [value]=\"file.progress\" size=\"tiny\"></nt-progress> </div> </div> ",
                host: {
                    'class': 'nt-file'
                },
                encapsulation: ViewEncapsulation.None,
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtFileComponent }
                ],
                animations: [
                    trigger('fadeOut', [
                        transition('* => void', fadeOut(.3))
                    ])
                ]
            },] },
];
/** @nocollapse */
NtFileComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_UPLOAD_HANDLER,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_FILE_ICONS,] }] }
];
NtFileComponent.propDecorators = {
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    accept: [{ type: Input }],
    maxSize: [{ type: Input }],
    maxFiles: [{ type: Input }],
    autoupload: [{ type: Input }],
    url: [{ type: Input }],
    name: [{ type: Input }],
    fileElement: [{ type: ViewChild, args: ['fileElement', { static: true },] }],
    error: [{ type: Output }],
    remove: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = DEFAULT_FILE_ICONS;
class NtFileModule {
    /**
     * @param {?=} icons
     * @return {?}
     */
    static forRoot(icons = DEFAULT_FILE_ICONS) {
        return {
            ngModule: NtFileModule,
            providers: [
                { provide: NT_FILE_ICONS, useValue: icons }
            ]
        };
    }
}
NtFileModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtProgressModule],
                exports: [NtFileComponent, NtFileSizePipe],
                declarations: [NtFileComponent, NtFileSizePipe],
                providers: [
                    { provide: NT_FILE_ICONS, useValue: ɵ0 }
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtFileModule, NtFile, NtFileComponent, NtFileSizePipe, NtFileAcceptError, NtFileSizeError, NtFileUploadError, DEFAULT_FILE_ICONS, NT_FILE_ICONS, NT_FILE_EXTENSIONS };
//# sourceMappingURL=file.js.map
