import { coerceNumberProperty, coerceBooleanProperty } from '@angular/cdk/coercion';
import { Directive, EventEmitter, Input, ChangeDetectionStrategy, ChangeDetectorRef, Component, ContentChild, NgZone, Optional, ViewEncapsulation, ContentChildren, Injectable, InjectionToken, Inject, Pipe, NgModule } from '@angular/core';
import { defer, Subject, of } from 'rxjs';
import { filter, switchMap, take, takeUntil } from 'rxjs/operators';
import { transition, trigger } from '@angular/animations';
import { FormControl, FormGroupDirective, NgForm } from '@angular/forms';
import { fadeIn, fadeOut } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
/**
 * An interface which allows a control to work inside of a `NtFormField`.
 * @abstract
 * @template T
 */
class NtFormFieldControl {
    /**
     *
     * @return {?}
     */
    focus() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormLabelWidthDirective {
    constructor() {
        this.widthChange = new EventEmitter();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set width(value) {
        this._width = coerceNumberProperty(value, 0);
        if (this._width > 0) {
            this.widthChange.next(this._width);
        }
    }
    /**
     * @return {?}
     */
    get width() { return this._width || 120; }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.widthChange.next();
        this.widthChange.complete();
    }
}
NtFormLabelWidthDirective.decorators = [
    { type: Directive, args: [{
                selector: 'form[ntFormLabelWidth]',
            },] },
];
NtFormLabelWidthDirective.propDecorators = {
    width: [{ type: Input, args: ['ntFormLabelWidth',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormOrientationDirective {
    constructor() {
        this.orientationChange = new EventEmitter();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set orientation(value) {
        this._orientation = value;
        this.orientationChange.next(this._orientation);
    }
    /**
     * @return {?}
     */
    get orientation() { return this._orientation; }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.orientationChange.next();
        this.orientationChange.complete();
    }
}
NtFormOrientationDirective.decorators = [
    { type: Directive, args: [{
                selector: 'form[ntFormOrientation]',
            },] },
];
NtFormOrientationDirective.propDecorators = {
    orientation: [{ type: Input, args: ['ntFormOrientation',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormFieldComponent {
    /**
     * @param {?} _ngZone
     * @param {?} _changeDetectorRef
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} _formLabelWidth
     * @param {?} _formOrientation
     */
    constructor(_ngZone, _changeDetectorRef, parentForm, parentFormGroup, _formLabelWidth, _formOrientation) {
        this._ngZone = _ngZone;
        this._changeDetectorRef = _changeDetectorRef;
        this._formLabelWidth = _formLabelWidth;
        this._formOrientation = _formOrientation;
        this._destroy = new Subject();
        /**
         * 表单可见性
         */
        this._labelVisible = true;
        this._selfLabelWidth = false;
        this._selfOrientation = false;
        this._markVisible = true;
        this._ngForm = null;
        /**
         * 表单宽度 （只在 horizontal 模式下起作用）
         */
        this._labelStyles = {};
        this._groupStyles = {};
        this._invalid = false;
        this.statusChanges = defer((/**
         * @return {?}
         */
        () => {
            if (this.control && this.ngControl) {
                return this.ngControl.statusChanges ? this.ngControl.statusChanges : of(null);
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            () => this.statusChanges)));
        }));
        this._ngForm = parentForm || parentFormGroup;
        if (this._formLabelWidth) {
            this._formLabelWidth.widthChange.pipe(takeUntil(this._destroy), filter((/**
             * @return {?}
             */
            () => !this._selfLabelWidth)))
                .subscribe((/**
             * @param {?} width
             * @return {?}
             */
            width => {
                this._labelWidth = width;
                this._setHorizontalStyles();
            }));
        }
        if (this._formOrientation) {
            this._formOrientation.orientationChange.pipe(takeUntil(this._destroy), filter((/**
             * @return {?}
             */
            () => !this._selfOrientation)))
                .subscribe((/**
             * @param {?} orientation
             * @return {?}
             */
            orientation => {
                this._orientation = orientation;
                this._setHorizontalStyles();
            }));
        }
        this.statusChanges.pipe(takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        () => this._validate()));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set labelVisible(value) { this._labelVisible = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get labelVisible() { return this._labelVisible; }
    /**
     * @param {?} value
     * @return {?}
     */
    set labelWidth(value) {
        this._labelWidth = coerceNumberProperty(value, 0);
        this._selfLabelWidth = this._labelWidth > 0;
        this._setHorizontalStyles();
    }
    /**
     * @return {?}
     */
    get labelWidth() { return this._labelWidth || 120; }
    /**
     * @param {?} value
     * @return {?}
     */
    set orientation(value) {
        this._orientation = value;
        this._selfOrientation = true;
        this._setHorizontalStyles();
    }
    /**
     * @return {?}
     */
    get orientation() { return this._orientation; }
    /**
     * @return {?}
     */
    get horizontal() { return this.orientation === 'horizontal'; }
    /**
     * @return {?}
     */
    get markVisible() { return this._markVisible; }
    /**
     * @param {?} value
     * @return {?}
     */
    set markVisible(value) {
        this._markVisible = coerceBooleanProperty(value);
    }
    /**
     * @return {?}
     */
    get required() {
        if (this.ngControl &&
            this.ngControl.control &&
            this.ngControl.control.validator) {
            /** @type {?} */
            const control = new FormControl();
            /** @type {?} */
            const validateResult = this.ngControl.control.validator(control);
            return validateResult && validateResult.hasOwnProperty('required');
        }
        if (this.control) {
            return !!this.control.required;
        }
        return false;
    }
    /**
     * @return {?}
     */
    get errors() {
        return this.control.ngControl ? this.control.ngControl.errors : null;
    }
    /**
     * @return {?}
     */
    get ngControl() { return this.control ? this.control.ngControl : null; }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this._ngForm && this.ngControl) {
            this._ngForm.ngSubmit.pipe(takeUntil(this._destroy)).subscribe((/**
             * @return {?}
             */
            () => this._validate()));
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
    }
    /**
     * @private
     * @return {?}
     */
    _validate() {
        if (this.ngControl) {
            this._invalid = !!this.ngControl.invalid;
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    _clearValidateMessage() {
        this._invalid = false;
    }
    /**
     * @private
     * @return {?}
     */
    _setHorizontalStyles() {
        if (this.labelWidth > 0 && this.horizontal) {
            this._labelStyles['width.px'] = this.labelWidth;
            this._groupStyles['margin-left.px'] = this.labelWidth;
        }
        else {
            delete this._labelStyles['width.px'];
            delete this._groupStyles['margin-left.px'];
        }
    }
}
NtFormFieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-form-field',
                template: `
    <label class="nt-form-label"
      *ngIf="labelVisible"
      [ngStyle]="_labelStyles"
      [class.required]="markVisible && required">
      {{label}}
    </label>
    <div class="nt-form-group" [ngStyle]="_groupStyles">
      <ng-content></ng-content>
      <span class="form-error is-visible" *ngIf="_invalid && errors" [@fade]="_invalid">
        {{ errors | ntFormError:label:messages }}
      </span>
    </div>
  `,
                animations: [
                    trigger('fade', [
                        transition('* => false', fadeOut(.15)),
                        transition('* => true', fadeIn(.15))
                    ])
                ],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-form-field',
                    '[class.nt-form-error]': '_invalid',
                    '[class.nt-form-horizontal]': 'horizontal'
                }
            },] },
];
/** @nocollapse */
NtFormFieldComponent.ctorParameters = () => [
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NtFormLabelWidthDirective, decorators: [{ type: Optional }] },
    { type: NtFormOrientationDirective, decorators: [{ type: Optional }] }
];
NtFormFieldComponent.propDecorators = {
    label: [{ type: Input }],
    labelVisible: [{ type: Input }],
    labelWidth: [{ type: Input }],
    messages: [{ type: Input }],
    orientation: [{ type: Input }],
    markVisible: [{ type: Input }],
    control: [{ type: ContentChild, args: [NtFormFieldControl, { static: false },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormAutofocusDirective {
    /**
     * @param {?} form
     * @param {?} formGroup
     */
    constructor(form, formGroup) {
        this._destory = new Subject();
        this.formContainer = form || formGroup;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this.formContainer) {
            this.formContainer.ngSubmit
                .pipe(takeUntil(this._destory))
                .subscribe((/**
             * @return {?}
             */
            () => {
                /** @type {?} */
                const field = this.fields.find((/**
                 * @param {?} field
                 * @return {?}
                 */
                field => !!field.ngControl && !!field.ngControl.invalid));
                if (field && field.control) {
                    field.control.focus();
                }
            }));
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destory.next();
        this._destory.complete();
    }
}
NtFormAutofocusDirective.decorators = [
    { type: Directive, args: [{
                selector: 'form[ntFormAutofocus]'
            },] },
];
/** @nocollapse */
NtFormAutofocusDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] }
];
NtFormAutofocusDirective.propDecorators = {
    fields: [{ type: ContentChildren, args: [NtFormFieldComponent, { descendants: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_TEMPLATES = {
    "required": "请填写{0}",
    "email": "{0}不是邮箱格式",
    "min": "{0}必须输入大于{1}的数字",
    "max": "{0}必须输入小于{1}的数字",
    "pattern": "{0}不是合法的数据",
    "maxlength": "{0}长度为最大{1}",
    "minlength": "{0}长度为最小{1}"
};
class NtFormValidationTransformer {
    /**
     * @param {?=} errors
     * @param {?=} label
     * @param {?=} messages
     * @return {?}
     */
    transform(errors, label, messages) {
        if (!errors) {
            return '';
        }
        /** @type {?} */
        const templates = Object.assign({}, DEFAULT_TEMPLATES, messages);
        if (errors.hasOwnProperty('required')) {
            return this._format(templates['required'], label);
        }
        else if (errors.hasOwnProperty('email')) {
            return this._format(templates['email'], label);
        }
        else if (errors.hasOwnProperty('min')) {
            return this._format(templates['min'], label, errors.min.min);
        }
        else if (errors.hasOwnProperty('max')) {
            return this._format(templates['max'], label, errors.max.max);
        }
        else if (errors.hasOwnProperty('minlength')) {
            return this._format(templates['minlength'], label, errors.minlength.requiredLength);
        }
        else if (errors.hasOwnProperty('maxlength')) {
            return this._format(templates['maxlength'], label, errors.maxlength.requiredLength);
        }
        else if (errors.hasOwnProperty('pattern')) {
            return this._format(templates['pattern'], label);
        }
        else {
            return '';
        }
    }
    /**
     * @protected
     * @param {?} template
     * @param {...?} args
     * @return {?}
     */
    _format(template, ...args) {
        return template.replace(/\{(\d+)\}/g, (/**
         * @param {?} match
         * @param {?} number
         * @return {?}
         */
        function (match, number) {
            return typeof args[number] !== 'undefined' ? args[number] : match;
        }));
    }
}
NtFormValidationTransformer.decorators = [
    { type: Injectable },
];
// export const NT_VALIDATION_TEMPLATE = new InjectionToken<NtValidationTemplate>('nt-validation-template');
/** @type {?} */
const NT_VALIDATION_TRANSFOMER = new InjectionToken('nt-validation-transformer');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormErrorPipe {
    /**
     * @param {?} _transformer
     */
    constructor(_transformer) {
        this._transformer = _transformer;
    }
    /**
     * @param {?} value
     * @param {...?} args
     * @return {?}
     */
    transform(value, ...args) {
        return this._transformer.transform(value, ...args);
    }
}
NtFormErrorPipe.decorators = [
    { type: Pipe, args: [{ name: 'ntFormError' },] },
];
/** @nocollapse */
NtFormErrorPipe.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [NT_VALIDATION_TRANSFOMER,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormsModule {
    /**
     * @param {?=} transformer
     * @return {?}
     */
    static forRoot(transformer) {
        return {
            ngModule: NtFormsModule,
            providers: [
                { provide: NT_VALIDATION_TRANSFOMER, useClass: transformer || NtFormValidationTransformer }
            ]
        };
    }
}
NtFormsModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [
                    NtFormFieldComponent,
                    NtFormAutofocusDirective,
                    NtFormLabelWidthDirective,
                    NtFormOrientationDirective,
                    NtFormErrorPipe
                ],
                declarations: [
                    NtFormFieldComponent,
                    NtFormAutofocusDirective,
                    NtFormLabelWidthDirective,
                    NtFormOrientationDirective,
                    NtFormErrorPipe
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtFormsModule, NtFormErrorPipe, NtFormFieldComponent, NtFormFieldControl, NtFormAutofocusDirective, NtFormLabelWidthDirective, NtFormOrientationDirective, DEFAULT_TEMPLATES, NtFormValidationTransformer, NT_VALIDATION_TRANSFOMER };
//# sourceMappingURL=forms.js.map
