import { InjectionToken, Component, EventEmitter, Inject, Input, Optional, Output, ViewEncapsulation, NgModule } from '@angular/core';
import { coerceNumberProperty } from '@angular/cdk/coercion';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPaginationConfig {
    constructor() {
        this.previousLabel = 'Previous';
        this.nextLabel = 'Next';
        this.size = 3;
        this.pageSize = 10;
    }
}
/** @type {?} */
const NT_PAGINATION_CONFIG = new InjectionToken('nt-pagination-config');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const PAGINATION_ELLIPSIS = '...';
class NtPaginationComponent {
    /**
     * @param {?=} defaultConfig
     */
    constructor(defaultConfig) {
        this._options = new NtPaginationConfig();
        this._totalPage = 1;
        this._pages = [1];
        this._total = 0;
        this._pageIndex = 1;
        this.pageChange = new EventEmitter();
        this._options = Object.assign({}, this._options, defaultConfig || {});
    }
    /**
     * @return {?}
     */
    get options() { return this._options; }
    /**
     * @param {?} value
     * @return {?}
     */
    set options(value) {
        if (typeof value === 'object') {
            this._options = Object.assign(this._options, value);
        }
    }
    /**
     * @return {?}
     */
    get totalPage() { return this._totalPage; }
    /**
     * @return {?}
     */
    get pages() { return this._pages; }
    /**
     * @param {?} value
     * @return {?}
     */
    set pageSize(value) { this._options.pageSize = coerceNumberProperty(value); }
    /**
     * @return {?}
     */
    get pageSize() { return this.options.pageSize; }
    /**
     * @param {?} value
     * @return {?}
     */
    set previousLabel(value) { this._options.previousLabel = value; }
    /**
     * @return {?}
     */
    get previousLabel() { return this.options.previousLabel; }
    /**
     * @param {?} value
     * @return {?}
     */
    set nextLabel(value) { this._options.nextLabel = value; }
    /**
     * @return {?}
     */
    get nextLabel() { return this.options.nextLabel; }
    /**
     * @param {?} value
     * @return {?}
     */
    set total(value) { this._total = coerceNumberProperty(value); this._build(); }
    /**
     * @return {?}
     */
    get total() { return this._total; }
    /**
     * @param {?} value
     * @return {?}
     */
    set pageIndex(value) { this._pageIndex = coerceNumberProperty(value, 1); this._build(); }
    /**
     * @return {?}
     */
    get pageIndex() { return this._pageIndex; }
    /**
     * @param {?} index
     * @return {?}
     */
    _pageChange(index) {
        this._pageIndex = coerceNumberProperty(index, 1);
        this.pageChange.emit(index);
    }
    /**
     * @private
     * @return {?}
     */
    _build() {
        this._totalPage = Math.ceil(this.total / this.options.pageSize);
        /** @type {?} */
        let pages = [1];
        if (this._totalPage > 1) {
            /** @type {?} */
            let start = this.pageIndex - this.options.size;
            /** @type {?} */
            let end = this.pageIndex + this.options.size;
            start = start < 2 ? 2 : start;
            end = end > this.totalPage - 1 ? this.totalPage - 1 : end;
            start - 2 >= 1 && (pages.push(PAGINATION_ELLIPSIS));
            pages = pages.concat(Array(end - start + 1).fill(start).map((/**
             * @param {?} value
             * @param {?} index
             * @return {?}
             */
            (value, index) => value + index)));
            end + 2 <= this.totalPage && (pages.push(PAGINATION_ELLIPSIS));
            this.totalPage > 1 && pages.push(this.totalPage);
        }
        this._pages = pages;
    }
}
NtPaginationComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-pagination, [nt-pagination]',
                template: "<ul class=\"pagination\"> <li class=\"pagination-previous disabled\" *ngIf=\"pageIndex <= 1\">{{ previousLabel }}</li> <li class=\"pagination-previous\" *ngIf=\"pageIndex > 1\"><a (click)=\"_pageChange(pageIndex - 1)\">{{ previousLabel }}</a></li> <ng-container *ngFor=\"let page of pages\"> <li *ngIf=\"page === pageIndex\" [class.current]=\"page === pageIndex\">{{ page }}</li> <li *ngIf=\"page !== pageIndex && page !== '...'\"><a (click)=\"_pageChange(page)\">{{ page }}</a></li> <li class=\"ellipsis\" *ngIf=\"page === '...'\"></li> </ng-container> <li class=\"pagination-next disabled\" *ngIf=\"pageIndex >= totalPage\">{{ nextLabel }}</li> <li class=\"pagination-next\" *ngIf=\"pageIndex < totalPage\"><a (click)=\"_pageChange(pageIndex + 1)\">{{ nextLabel }}</a></li> </ul> ",
                encapsulation: ViewEncapsulation.None
            },] },
];
/** @nocollapse */
NtPaginationComponent.ctorParameters = () => [
    { type: NtPaginationConfig, decorators: [{ type: Optional }, { type: Inject, args: [NT_PAGINATION_CONFIG,] }] }
];
NtPaginationComponent.propDecorators = {
    options: [{ type: Input }],
    pageSize: [{ type: Input }],
    previousLabel: [{ type: Input }],
    nextLabel: [{ type: Input }],
    total: [{ type: Input }],
    pageIndex: [{ type: Input }],
    pageChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPaginationModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static withConfig(config) {
        return {
            ngModule: NtPaginationModule,
            providers: [
                { provide: NT_PAGINATION_CONFIG, useValue: config || new NtPaginationConfig() }
            ]
        };
    }
}
NtPaginationModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtPaginationComponent],
                declarations: [NtPaginationComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtPaginationModule, PAGINATION_ELLIPSIS, NtPaginationComponent, NtPaginationConfig, NT_PAGINATION_CONFIG };
//# sourceMappingURL=pagination.js.map
