import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { Component, ElementRef, EventEmitter, Input, Output, TemplateRef, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import { NtOverlayComponent, NtOverlayPosition, NtOverlayModule } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';
import { NtButtonModule } from '@ng-tangram/components/button';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPopConfirmComponent {
    /**
     * @param {?} _elementRef
     */
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._title = '';
        this.position = NtOverlayPosition.Top;
        this.confirmText = '确认';
        this.cancelText = '取消';
        this.confirm = new EventEmitter();
        this.cancel = new EventEmitter();
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(this._elementRef);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set title(value) { this._title = value; }
    /**
     * @return {?}
     */
    get title() { return this._title; }
    /**
     * @param {?} value
     * @return {?}
     */
    set template(value) { this._template = value; }
    /**
     * @return {?}
     */
    get template() { return this._template; }
    /**
     * @param {?} value
     * @return {?}
     */
    set popconfirm(value) {
        if (value instanceof TemplateRef) {
            this._template = value;
        }
        else {
            this._title = value;
            this._template = null;
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const change = changes.title || changes.template || changes.popover;
        if (change && !change.firstChange) {
            /** 在内容更新之后提示框的位置需要更新，需要延迟执行，因为这时候画面还未渲染 */
            Promise.resolve().then((/**
             * @return {?}
             */
            () => {
                this.overlay.cdkConnectedOverlay.overlayRef.updatePosition();
            }));
        }
    }
    /**
     * @param {?} isConfirm
     * @return {?}
     */
    _closeOverlay(isConfirm) {
        (isConfirm ? this.confirm : this.cancel).emit();
        this.overlay.hide();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtPopConfirmComponent.decorators = [
    { type: Component, args: [{
                selector: '[nt-popconfirm]',
                template: "<ng-content></ng-content> <nt-overlay trigger=\"click\" arrow [origin]=\"origin\" [position]=\"position\" (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\"> <div class=\"nt-popconfirm\"> <div class=\"nt-popconfirm-title\" *ngIf=\"template\"><ng-container *ngTemplateOutlet=\"template\"></ng-container></div> <div class=\"nt-popconfirm-title\" *ngIf=\"!template\" [innerHtml]=\"title\"></div> <div class=\"nt-popconfirm-action\"> <button nt-button color=\"secondary\" size=\"tiny\" (click)=\"_closeOverlay(false)\">{{cancelText}}</button> <button nt-button size=\"tiny\" (click)=\"_closeOverlay(true)\">{{confirmText}}</button> </div> </div> </nt-overlay> ",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '(click)': 'overlay.click()'
                }
            },] },
];
/** @nocollapse */
NtPopConfirmComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtPopConfirmComponent.propDecorators = {
    title: [{ type: Input }],
    template: [{ type: Input }],
    popconfirm: [{ type: Input, args: ['nt-popconfirm',] }],
    position: [{ type: Input }],
    confirmText: [{ type: Input }],
    cancelText: [{ type: Input }],
    confirm: [{ type: Output }],
    cancel: [{ type: Output }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPopConfirmModule {
}
NtPopConfirmModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtOverlayModule, NtButtonModule],
                exports: [NtPopConfirmComponent],
                declarations: [NtPopConfirmComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtPopConfirmModule, NtPopConfirmComponent };
//# sourceMappingURL=popconfirm.js.map
