import { Subject } from 'rxjs';
import { ContentObserver, ObserversModule } from '@angular/cdk/observers';
import { Component, ElementRef, Inject, InjectionToken, ViewEncapsulation, EventEmitter, Input, Output, TemplateRef, ViewChild, NgModule } from '@angular/core';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { NtOverlayComponent, NtOverlayPosition, NtOverlayModule } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';
import { NtDropdownModule } from '@ng-tangram/components/dropdown';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_POPOVER_PARENT_COMPONENT = new InjectionToken('nt-popover-parent-component');
class NtPopoverPaneComponent {
    /**
     * @param {?} _contentObserver
     * @param {?} _elementRef
     * @param {?} _parent
     */
    constructor(_contentObserver, _elementRef, _parent) {
        this._contentObserver = _contentObserver;
        this._elementRef = _elementRef;
        this._parent = _parent;
        this._contentChange = new Subject();
    }
    /**
     * @return {?}
     */
    get contentChanged() {
        return this._contentChange.asObservable();
    }
    /**
     * @return {?}
     */
    get textContent() {
        return (this._elementRef.nativeElement.textContent || '').trim();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._contentSubscription = this._contentObserver
            .observe(this._elementRef)
            .subscribe((/**
         * @return {?}
         */
        () => this._checkContentChange()));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._contentSubscription) {
            this._contentSubscription.unsubscribe();
        }
    }
    /**
     * @private
     * @return {?}
     */
    _checkContentChange() {
        if (this._parent.overlay.isOpen) {
            /** @type {?} */
            const cdkConnectedOverlay = this._parent.overlay.cdkConnectedOverlay;
            if (cdkConnectedOverlay && cdkConnectedOverlay.overlayRef) {
                cdkConnectedOverlay.overlayRef.updatePosition();
            }
        }
    }
}
NtPopoverPaneComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-popover-pane, [nt-popover-pane]',
                template: `
    <ng-content></ng-content>
  `,
                encapsulation: ViewEncapsulation.None
            },] },
];
/** @nocollapse */
NtPopoverPaneComponent.ctorParameters = () => [
    { type: ContentObserver },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Inject, args: [NT_POPOVER_PARENT_COMPONENT,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPopoverComponent {
    /**
     * @param {?} _elementRef
     */
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._title = '';
        this.position = NtOverlayPosition.Top;
        this.confirm = new EventEmitter();
        this.cancel = new EventEmitter();
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(this._elementRef);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set title(value) { this._title = value; }
    /**
     * @return {?}
     */
    get title() { return this._title; }
    /**
     * @param {?} value
     * @return {?}
     */
    set template(value) { this._template = value; }
    /**
     * @return {?}
     */
    get template() { return this._template; }
    /**
     * @param {?} value
     * @return {?}
     */
    set popover(value) {
        if (value instanceof TemplateRef) {
            this._template = value;
        }
        else {
            this._title = value;
            this._template = null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtPopoverComponent.decorators = [
    { type: Component, args: [{
                selector: '[nt-popover]',
                template: "<ng-content></ng-content> <nt-overlay trigger=\"click\" arrow [origin]=\"origin\" [position]=\"position\" (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\"> <div class=\"nt-popover\"> <div class=\"nt-popover-title\" *ngIf=\"template\"><ng-container *ngTemplateOutlet=\"template\"></ng-container></div> <div class=\"nt-popover-title\" *ngIf=\"!template\" [innerHtml]=\"title\"></div> <div class=\"nt-popover-pane\"> <ng-content select=\"nt-popover-pane\"></ng-content> </div> </div> </nt-overlay> ",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '(click)': 'overlay.click()'
                },
                providers: [
                    { provide: NT_POPOVER_PARENT_COMPONENT, useExisting: NtPopoverComponent }
                ]
            },] },
];
/** @nocollapse */
NtPopoverComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtPopoverComponent.propDecorators = {
    title: [{ type: Input }],
    template: [{ type: Input }],
    popover: [{ type: Input, args: ['nt-popover',] }],
    position: [{ type: Input }],
    confirm: [{ type: Output }],
    cancel: [{ type: Output }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtPopoverModule {
}
NtPopoverModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ObserversModule, NtOverlayModule, NtDropdownModule],
                exports: [NtPopoverComponent, NtPopoverPaneComponent],
                declarations: [NtPopoverComponent, NtPopoverPaneComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtPopoverModule, NtPopoverComponent, NT_POPOVER_PARENT_COMPONENT, NtPopoverPaneComponent };
//# sourceMappingURL=popover.js.map
