import { ChangeDetectionStrategy, Component, Input, ViewEncapsulation, ChangeDetectorRef, Attribute, Self, Optional, Output, EventEmitter, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
let uniqueId = 0;
/**
 * @template T
 */
class NtSwitchChange {
    /**
     * @param {?} source
     * @param {?} checked
     */
    constructor(source, checked) {
        this.source = source;
        this.checked = checked;
    }
}
/**
 * @template T
 */
class NtSwitchComponent {
    /**
     * @param {?} _changeDetectorRef
     * @param {?} tabIndex
     * @param {?} ngControl
     */
    constructor(_changeDetectorRef, tabIndex, ngControl) {
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this.id = `nt-switch-${uniqueId++}`;
        this._disabled = false;
        this._checked = false;
        this._circle = false;
        this.change = new EventEmitter();
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this.tabIndex = parseInt(tabIndex) || 0;
    }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) {
        if (value !== this.disabled) {
            this._disabled = coerceBooleanProperty(value);
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    get checked() { return this._checked; }
    /**
     * @param {?} value
     * @return {?}
     */
    set checked(value) {
        if (value !== this.checked) {
            this._checked = coerceBooleanProperty(value);
            this._changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    get circle() { return this._circle; }
    /**
     * @param {?} value
     * @return {?}
     */
    set circle(value) {
        if (value !== this.circle) {
            this._circle = coerceBooleanProperty(value);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._checked = !!value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onInputClick(event) {
        event.stopPropagation();
        if (!this.disabled) {
            this._checked = !this._checked;
            this._onTouched();
            this._emitChangeEvent();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _onInteractionEvent(event) {
        // We always have to stop propagation on the change event.
        // Otherwise the change event, from the input element, will bubble up and
        // emit its event object to the `change` output.
        event.stopPropagation();
    }
    /**
     * @private
     * @return {?}
     */
    _emitChangeEvent() {
        /** @type {?} */
        const event = new NtSwitchChange(this, this.checked);
        this._onChange(this.checked);
        this.change.emit(event);
    }
}
NtSwitchComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-switch',
                template: "<input class=\"switch-input\" type=\"checkbox\" [attr.id]=\"id\" [tabIndex]=\"tabIndex\" [checked]=\"checked\" [disabled]=\"disabled\" (change)=\"_onInteractionEvent($event)\" (click)=\"_onInputClick($event)\"> <label class=\"switch-paddle\" [attr.for]=\"id\"></label> ",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'switch nt-switch',
                    '[class.nt-switch-checked]': 'checked',
                    '[class.nt-switch-disabled]': 'disabled',
                    '[class.nt-switch-circle]': 'circle'
                }
            },] },
];
/** @nocollapse */
NtSwitchComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] }
];
NtSwitchComponent.propDecorators = {
    disabled: [{ type: Input }],
    checked: [{ type: Input }],
    circle: [{ type: Input }],
    change: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtSwitchModule {
}
NtSwitchModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtSwitchComponent],
                declarations: [NtSwitchComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtSwitchChange, NtSwitchComponent, NtSwitchModule };
//# sourceMappingURL=switch.js.map
