import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CdkCell, CdkCellDef, CdkColumnDef, CdkFooterCell, CdkFooterCellDef, CdkHeaderCell, CdkHeaderCellDef, CDK_ROW_TEMPLATE, CdkFooterRow, CdkFooterRowDef, CdkHeaderRow, CdkHeaderRowDef, CdkRow, CdkRowDef, CDK_TABLE_TEMPLATE, CdkTable, CdkTableModule, DataSource } from '@angular/cdk/table';
import { Directive, ElementRef, EventEmitter, Input, ChangeDetectionStrategy, Component, ViewEncapsulation, Attribute, ChangeDetectorRef, ContentChildren, IterableDiffers, NgZone, Optional, Output, Inject, NgModule } from '@angular/core';
import { defer, merge, Subject, BehaviorSubject, combineLatest, Subscription } from 'rxjs';
import { filter, startWith, switchMap, take, takeUntil, map } from 'rxjs/operators';
import { Directionality } from '@angular/cdk/bidi';
import { SelectionModel } from '@angular/cdk/collections';
import { Platform } from '@angular/cdk/platform';
import { DOCUMENT, CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const NtColumnSort = {
    ASC: 'asc',
    DESC: 'desc',
    NONE: '',
};
class NtColumnSortChange {
    /**
     * @param {?=} isUserInput
     * @param {?=} column
     * @param {?=} sort
     */
    constructor(isUserInput = false, column, sort) {
        this.isUserInput = isUserInput;
        this.column = column;
        this.sort = sort;
    }
}
/**
 * table 列单元格定义指令
 * \@description ntColumnCellDef 将在 > 0.5.x 版本中弃用
 */
class NtCellDefDirective extends CdkCellDef {
}
NtCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntCellDef], [ntColumnCellDef]',
                providers: [{ provide: CdkCellDef, useExisting: NtCellDefDirective }]
            },] },
];
/**
 * table 头部单元格定义指令
 * \@description ntColumnHeaderDef 将在 > 0.5.x 版本中弃用
 */
class NtHeaderCellDefDirective extends CdkHeaderCellDef {
}
NtHeaderCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntHeaderCellDef], [ntColumnHeaderDef]',
                providers: [{ provide: CdkHeaderCellDef, useExisting: NtHeaderCellDefDirective }]
            },] },
];
/**
 * table 底部单元格定义指令
 * \@description ntColumnFooterDef 将在 > 0.5.x 版本中弃用
 */
class NtFooterCellDefDirective extends CdkFooterCellDef {
}
NtFooterCellDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntFooterCellDef], [ntColumnFooterDef]',
                providers: [{ provide: CdkFooterCellDef, useExisting: NtFooterCellDefDirective }]
            },] },
];
/**
 * table 列定义指令
 */
class NtColumnDirective extends CdkColumnDef {
    constructor() {
        super(...arguments);
        this._sortable = false;
        this.sort = NtColumnSort.NONE;
        this._sortChange = new EventEmitter();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set ntColumn(value) { this.name = value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set sortable(value) { this._sortable = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get sortable() { return this._sortable; }
    /**
     * 排序操作
     * @param {?=} isUserInput
     * @return {?}
     */
    sorting(isUserInput = false) {
        if (this.sortable) {
            /** 按照 升 -> 降 -> 无 的循环改变排序 */
            switch (this.sort) {
                case NtColumnSort.ASC:
                    this.sort = NtColumnSort.DESC;
                    break;
                case NtColumnSort.DESC:
                    this.sort = NtColumnSort.NONE;
                    break;
                default:
                    this.sort = NtColumnSort.ASC;
                    break;
            }
            this._sortChange.emit(new NtColumnSortChange(isUserInput, this.name, this.sort));
        }
    }
}
NtColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-column, [nt-column]',
                providers: [{ provide: CdkColumnDef, useExisting: NtColumnDirective }],
            },] },
];
NtColumnDirective.propDecorators = {
    ntColumn: [{ type: Input, args: ['nt-column',] }],
    name: [{ type: Input }],
    sticky: [{ type: Input }],
    stickyEnd: [{ type: Input }],
    sortable: [{ type: Input }]
};
/**
 * table 头部单元格结构指令
 * \@description nt-column-header 将在 > 0.5.x 版本中弃用
 *
 */
class NtHeaderCellDirective extends CdkHeaderCell {
    /**
     * @param {?} columnDef
     * @param {?} elementRef
     */
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        this.columnDef = columnDef;
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtHeaderCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-header-cell, nt-column-header, th[nt-header-cell], th[nt-column-header]',
                host: {
                    'class': 'nt-header-cell',
                    '[class.nt-column-sortable]': 'columnDef.sortable',
                    '[class.asc]': 'columnDef.sort === "asc"',
                    '[class.desc]': 'columnDef.sort === "desc"',
                    '(click)': 'columnDef.sorting(true)',
                    'role': 'columnheader',
                },
            },] },
];
/** @nocollapse */
NtHeaderCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];
/**
 * table 底部单元格结构指令
 * \@description nt-column-footer 将在 > 0.5.x 版本中弃用
 *
 */
class NtFooterCellDirective extends CdkFooterCell {
    /**
     * @param {?} columnDef
     * @param {?} elementRef
     */
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtFooterCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-footer-cell, nt-column-footer, td[nt-footer-cell], td[nt-column-footer]',
                host: {
                    'class': 'nt-footer-cell',
                    'role': 'gridcell',
                },
            },] },
];
/** @nocollapse */
NtFooterCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];
/**
 * table 单元格结构指令
 * \@description nt-column-cell 将在 > 0.5.x 版本中弃用
 */
class NtCellDirective extends CdkCell {
    /**
     * @param {?} columnDef
     * @param {?} elementRef
     */
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
}
NtCellDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-cell, nt-column-cell, td[nt-cell], td[nt-column-cell]',
                host: {
                    'class': 'nt-cell',
                    'role': 'gridcell',
                },
            },] },
];
/** @nocollapse */
NtCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtHeaderRowDefDirective extends CdkHeaderRowDef {
}
NtHeaderRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntHeaderRowDef]',
                providers: [{ provide: CdkHeaderRowDef, useExisting: NtHeaderRowDefDirective }],
                inputs: ['columns: ntHeaderRowDef', 'sticky: ntHeaderRowDefSticky'],
            },] },
];
class NtFooterRowDefDirective extends CdkFooterRowDef {
}
NtFooterRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntFooterRowDef]',
                providers: [{ provide: CdkFooterRowDef, useExisting: NtFooterRowDefDirective }],
                inputs: ['columns: ntFooterRowDef', 'sticky: ntFooterRowDefSticky'],
            },] },
];
/**
 * @template T
 */
class NtRowDefDirective extends CdkRowDef {
}
NtRowDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntRowDef]',
                providers: [{ provide: CdkRowDef, useExisting: NtRowDefDirective }],
                inputs: ['columns: ntRowDefColumns', 'when: ntRowDefWhen'],
            },] },
];
class NtHeaderRowComponent extends CdkHeaderRow {
}
NtHeaderRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-header-row, tr[nt-header-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-header-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntHeaderRow',
                providers: [{ provide: CdkHeaderRow, useExisting: NtHeaderRowComponent }],
            },] },
];
class NtFooterRowComponent extends CdkFooterRow {
}
NtFooterRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-footer-row, tr[nt-footer-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-footer-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntFooterRow',
                providers: [{ provide: CdkFooterRow, useExisting: NtFooterRowComponent }],
            },] },
];
class NtRowComponent extends CdkRow {
}
NtRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-row, tr[nt-row]',
                template: CDK_ROW_TEMPLATE,
                host: {
                    'class': 'nt-row',
                    'role': 'row',
                },
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                exportAs: 'ntRow',
                providers: [{ provide: CdkRow, useExisting: NtRowComponent }],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTableComponent extends CdkTable {
    /**
     * @param {?} _ngZone
     * @param {?} _differs
     * @param {?} _changeDetectorRef
     * @param {?} _elementRef
     * @param {?} role
     * @param {?} _dir
     * @param {?} _document
     * @param {?} _platform
     */
    constructor(_ngZone, _differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform) {
        super(_differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform);
        this._ngZone = _ngZone;
        this._differs = _differs;
        this._changeDetectorRef = _changeDetectorRef;
        this._elementRef = _elementRef;
        this._dir = _dir;
        this._selectionModel = new SelectionModel(true, undefined, false);
        this._multiSortable = false;
        this._selectable = false;
        this._destroy = new Subject();
        this.columSortChanges = defer((/**
         * @return {?}
         */
        () => {
            if (this._contentColumns) {
                return merge(...this._contentColumns.map((/**
                 * @param {?} column
                 * @return {?}
                 */
                column => column._sortChange)));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            () => this.columSortChanges)));
        }));
        this.selectedChange = new EventEmitter();
        this.sortChange = new EventEmitter();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set selectable(value) { this._selectable = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get selectable() { return this._selectable; }
    /**
     * @return {?}
     */
    get isAllSelected() { return false; }
    /**
     * @return {?}
     */
    get selected() { return this._selectionModel.selected; }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._contentColumns.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        () => {
            this._resetOptions();
            this._changeDetectorRef.markForCheck();
        }));
    }
    /**
     * @return {?}
     */
    selectAll() {
        if (!this.isAllSelected) {
            // this._selectionModel.select(...this.dataSource);
        }
        else {
            this._selectionModel.clear();
        }
        this.selectedChange.emit(this._selectionModel.selected);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    select(item) {
        /** @type {?} */
        const wasSelected = this.checkSelected(item);
        if (wasSelected) {
            this._selectionModel.deselect(item);
        }
        else {
            this._selectionModel.select(item);
        }
        this.selectedChange.emit(this._selectionModel.selected);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    ngOnChanges(change) {
        if (change && change.dataSource && !change.dataSource.firstChange) {
            this._selectionModel.clear();
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    checkSelected(item) {
        return this._selectionModel.isSelected(item);
    }
    /**
     * @private
     * @param {?=} filter
     * @return {?}
     */
    _clearSort(filter$$1) {
        this._contentColumns
            .filter((/**
         * @param {?} column
         * @return {?}
         */
        column => column.name !== (filter$$1 ? filter$$1.column : '')))
            .forEach((/**
         * @param {?} column
         * @return {?}
         */
        column => column.sort = NtColumnSort.NONE));
        this._changeDetectorRef.markForCheck();
    }
    /**
     * @private
     * @return {?}
     */
    _resetOptions() {
        /** @type {?} */
        const changedOrDestroyed = merge(this._contentColumns.changes, this._destroy);
        this.columSortChanges
            .pipe(takeUntil(changedOrDestroyed), filter((/**
         * @param {?} event
         * @return {?}
         */
        event => event.isUserInput)))
            .subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (!this._multiSortable) {
                this._clearSort(event);
            }
            this.sortChange.emit(event);
        }));
        merge(...this._contentColumns.map((/**
         * @param {?} column
         * @return {?}
         */
        column => column._sortChange)))
            .pipe(takeUntil(changedOrDestroyed))
            .subscribe((/**
         * @return {?}
         */
        () => {
            this._changeDetectorRef.markForCheck();
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        super.ngOnDestroy();
    }
}
NtTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-table, table[nt-table]',
                template: CDK_TABLE_TEMPLATE,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-table'
                }
            },] },
];
/** @nocollapse */
NtTableComponent.ctorParameters = () => [
    { type: NgZone },
    { type: IterableDiffers },
    { type: ChangeDetectorRef },
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['role',] }] },
    { type: Directionality, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Platform }
];
NtTableComponent.propDecorators = {
    selectable: [{ type: Input }],
    _contentColumns: [{ type: ContentChildren, args: [NtColumnDirective,] }],
    selectedChange: [{ type: Output }],
    sortChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtTableResizable {
    constructor() { }
}
NtTableResizable.decorators = [
    { type: Directive, args: [{
                selector: 'table[ntTableResizable], nt-table[ntTableResizable]'
            },] },
];
/** @nocollapse */
NtTableResizable.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_CELL_DECLARATIONS = [
    NtCellDefDirective,
    NtHeaderCellDefDirective,
    NtFooterCellDefDirective,
    NtColumnDirective,
    NtHeaderCellDirective,
    NtFooterCellDirective,
    NtCellDirective
];
/** @type {?} */
const NT_ROW_DECLARATIONS = [
    NtHeaderRowDefDirective,
    NtFooterRowDefDirective,
    NtRowDefDirective,
    NtHeaderRowComponent,
    NtFooterRowComponent,
    NtRowComponent
];
class NtTableModule {
}
NtTableModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, CdkTableModule],
                exports: [
                    NtTableComponent,
                    NtTableResizable,
                    ...NT_CELL_DECLARATIONS,
                    ...NT_ROW_DECLARATIONS
                ],
                declarations: [
                    NtTableComponent,
                    NtTableResizable,
                    ...NT_CELL_DECLARATIONS,
                    ...NT_ROW_DECLARATIONS
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTableDataSource extends DataSource {
    /**
     * @param {?=} initialData
     */
    constructor(initialData = []) {
        super();
        /**
         * Stream emitting render data to the table (depends on ordered data changes).
         */
        this._renderData = new BehaviorSubject([]);
        /**
         * Stream that emits when a new filter string is set on the data source.
         */
        this._filter = new BehaviorSubject('');
        /**
         * Subscription to the changes that should trigger an update to the table's rendered rows, such
         * as filtering, sorting, pagination, or base data changes.
         */
        this._renderChangesSubscription = Subscription.EMPTY;
        /**
         * Checks if a data object matches the data source's filter string. By default, each data object
         * is converted to a string of its properties and returns true if the filter has
         * at least one occurrence in that string. By default, the filter string has its whitespace
         * trimmed and the match is case-insensitive. May be overridden for a custom implementation of
         * filter matching.
         * @param data Data object used to check against the filter.
         * @param filter Filter string that has been set on the data source.
         * @return Whether the filter matches against the data
         */
        this.filterPredicate = (/**
         * @param {?} data
         * @param {?} filter
         * @return {?}
         */
        (data, filter$$1) => {
            // Transform the data into a lowercase string of all property values.
            /** @type {?} */
            const accumulator = (/**
             * @param {?} currentTerm
             * @param {?} key
             * @return {?}
             */
            (currentTerm, key) => currentTerm + data[key]);
            /** @type {?} */
            const dataStr = Object.keys(data).reduce(accumulator, '').toLowerCase();
            // Transform the filter by converting it to lowercase and removing whitespace.
            /** @type {?} */
            const transformedFilter = filter$$1.trim().toLowerCase();
            return dataStr.indexOf(transformedFilter) !== -1;
        });
        this._data = new BehaviorSubject(initialData);
        this._updateChangeSubscription();
    }
    /**
     * Array of data that should be rendered by the table, where each object represents one row.
     * @return {?}
     */
    get data() { return this._data.value; }
    /**
     * @param {?} data
     * @return {?}
     */
    set data(data) { this._data.next(data); }
    /**
     * Filter term that should be used to filter out objects from the data array. To override how
     * data objects match to this filter string, provide a custom function for filterPredicate.
     * @return {?}
     */
    get filter() { return this._filter.value; }
    /**
     * @param {?} filter
     * @return {?}
     */
    set filter(filter$$1) { this._filter.next(filter$$1); }
    /**
     * Subscribe to changes that should trigger an update to the table's rendered rows. When the
     * changes occur, process the current state of the filter, sort, and pagination along with
     * the provided base data and send it to the table for rendering.
     * @return {?}
     */
    _updateChangeSubscription() {
        /** @type {?} */
        const dataStream = this._data;
        // Watch for base data or filter changes to provide a filtered set of data.
        /** @type {?} */
        const filteredData = combineLatest(dataStream, this._filter)
            .pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        ([data]) => this._filterData(data))));
        // Watched for paged data changes and send the result to the table to render.
        this._renderChangesSubscription.unsubscribe();
        this._renderChangesSubscription = filteredData.subscribe((/**
         * @param {?} data
         * @return {?}
         */
        data => this._renderData.next(data)));
    }
    /**
     * Returns a filtered data array where each filter object contains the filter string within
     * the result of the filterTermAccessor function. If no filter is set, returns the data array
     * as provided.
     * @param {?} data
     * @return {?}
     */
    _filterData(data) {
        // If there is a filter string, filter out data that does not contain it.
        // Each data object is converted to a string using the function defined by filterTermAccessor.
        // May be overridden for customization.
        this.filteredData =
            !this.filter ? data : data.filter((/**
             * @param {?} obj
             * @return {?}
             */
            obj => this.filterPredicate(obj, this.filter)));
        return this.filteredData;
    }
    /**
     * Used by the NtTable. Called when it connects to the data source.
     * \@docs-private
     * @return {?}
     */
    connect() { return this._renderData; }
    /**
     * Used by the NtTable. Called when it is destroyed. No-op.
     * \@docs-private
     * @return {?}
     */
    disconnect() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtTableModule, NtTableComponent, NtTableDataSource, NtColumnSort, NtColumnSortChange, NtCellDefDirective, NtHeaderCellDefDirective, NtFooterCellDefDirective, NtColumnDirective, NtHeaderCellDirective, NtFooterCellDirective, NtCellDirective, NtHeaderRowDefDirective, NtFooterRowDefDirective, NtRowDefDirective, NtHeaderRowComponent, NtFooterRowComponent, NtRowComponent, NtTableResizable as ɵa };
//# sourceMappingURL=table.js.map
