import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { Component, ElementRef, EventEmitter, Input, Output, TemplateRef, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import { NtOverlayComponent, NtOverlayPosition, NtOverlayModule } from '@ng-tangram/components/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtTooltipComponent {
    /**
     * @param {?} _elementRef
     */
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this._title = '';
        this.position = NtOverlayPosition.Top;
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this.origin = new CdkOverlayOrigin(_elementRef);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set title(value) { this._title = value; }
    /**
     * @return {?}
     */
    get title() { return this._title; }
    /**
     * @param {?} value
     * @return {?}
     */
    set template(value) { this._template = value; }
    /**
     * @return {?}
     */
    get template() { return this._template; }
    /**
     * @param {?} value
     * @return {?}
     */
    set tooltip(value) {
        if (value instanceof TemplateRef) {
            this._template = value;
        }
        else {
            this._title = value;
            this._template = null;
        }
    }
    /**
     * @return {?}
     */
    get _isDirective() {
        /** @type {?} */
        const attributes = this._elementRef.nativeElement.attributes;
        return attributes && attributes['nt-tooltip'];
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        /** @type {?} */
        const change = changes.title || changes.template || changes.tooltip;
        if (change && !change.firstChange) {
            /** 在内容更新之后提示框的位置需要更新，需要延迟执行，因为这时候画面还未渲染 */
            Promise.resolve().then((/**
             * @return {?}
             */
            () => {
                this.overlay.cdkConnectedOverlay.overlayRef.updatePosition();
            }));
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterOpen(event) {
        this.afterOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _afterClosed(event) {
        this.afterClosed.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeOpen(event) {
        this.beforeOpen.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _beforeClosed(event) {
        this.beforeClosed.next(event);
    }
    /**
     * @param {?} change
     * @return {?}
     */
    _positionChange(change) {
        this.positionChange.next(change);
    }
}
NtTooltipComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-tooltip, [nt-tooltip]',
                template: "<ng-content></ng-content> <nt-overlay trigger=\"hover\" overlayClass=\"tootip-overlay\" arrow [origin]=\"origin\" [position]=\"position\" (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\"> <div class=\"tooltip\" *ngIf=\"template\"> <ng-container *ngTemplateOutlet=\"template\"></ng-container> </div> <div class=\"tooltip\" *ngIf=\"!template\" [innerHtml]=\"title\"></div> </nt-overlay> ",
                encapsulation: ViewEncapsulation.None,
                host: {
                    '[class.tooltip-trigger]': '!_isDirective',
                    '(mouseenter)': 'overlay.onMouseEnter()',
                    '(mouseleave)': 'overlay.onMouseLeave()'
                }
            },] },
];
/** @nocollapse */
NtTooltipComponent.ctorParameters = () => [
    { type: ElementRef }
];
NtTooltipComponent.propDecorators = {
    title: [{ type: Input }],
    template: [{ type: Input }],
    tooltip: [{ type: Input, args: ['nt-tooltip',] }],
    position: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtTooltipModule {
}
NtTooltipModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, NtOverlayModule],
                exports: [NtTooltipComponent],
                declarations: [NtTooltipComponent]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtTooltipModule, NtTooltipComponent };
//# sourceMappingURL=tooltip.js.map
