import { BehaviorSubject, merge } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { DataSource } from '@angular/cdk/collections';
import { CdkTreeNodeOutlet, CdkNestedTreeNode, CdkTree, CdkTreeNode, CdkTreeNodeDef, CdkTreeNodePadding, CdkTreeNodeToggle, CdkTreeModule } from '@angular/cdk/tree';
import { Directive, ViewContainerRef, Attribute, ContentChildren, ElementRef, Input, IterableDiffers, ChangeDetectionStrategy, Component, ViewEncapsulation, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTreeNestedDataSource extends DataSource {
    constructor() {
        super(...arguments);
        this._data = new BehaviorSubject([]);
    }
    /**
     * @return {?}
     */
    get data() { return this._data.value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set data(value) { this._data.next(value); }
    /**
     * @param {?} collectionViewer
     * @return {?}
     */
    connect(collectionViewer) {
        return merge(...[collectionViewer.viewChange, this._data])
            .pipe(map((/**
         * @return {?}
         */
        () => {
            return this.data;
        })));
    }
    /**
     * @return {?}
     */
    disconnect() {
        // no op
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T, F
 */
class NtTreeFlattener {
    /**
     * @param {?} transformFunction
     * @param {?} getLevel
     * @param {?} isExpandable
     * @param {?} getChildren
     */
    constructor(transformFunction, getLevel, isExpandable, getChildren) {
        this.transformFunction = transformFunction;
        this.getLevel = getLevel;
        this.isExpandable = isExpandable;
        this.getChildren = getChildren;
    }
    /**
     * @param {?} node
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    _flattenNode(node, level, resultNodes, parentMap) {
        /** @type {?} */
        const flatNode = this.transformFunction(node, level);
        resultNodes.push(flatNode);
        if (this.isExpandable(flatNode)) {
            /** @type {?} */
            const childrenNodes = this.getChildren(node);
            if (Array.isArray(childrenNodes)) {
                this._flattenChildren(childrenNodes, level, resultNodes, parentMap);
            }
            else {
                childrenNodes.pipe(take(1)).subscribe((/**
                 * @param {?} children
                 * @return {?}
                 */
                children => {
                    this._flattenChildren(children, level, resultNodes, parentMap);
                }));
            }
        }
        return resultNodes;
    }
    /**
     * @param {?} children
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    _flattenChildren(children, level, resultNodes, parentMap) {
        children.forEach((/**
         * @param {?} child
         * @param {?} index
         * @return {?}
         */
        (child, index) => {
            /** @type {?} */
            let childParentMap = parentMap.slice();
            childParentMap.push(index !== children.length - 1);
            this._flattenNode(child, level + 1, resultNodes, childParentMap);
        }));
    }
    /**
     * @param {?} structuredData
     * @return {?}
     */
    flattenNodes(structuredData) {
        /** @type {?} */
        let resultNodes = [];
        structuredData.forEach((/**
         * @param {?} node
         * @return {?}
         */
        node => this._flattenNode(node, 0, resultNodes, [])));
        return resultNodes;
    }
    /**
     * @param {?} nodes
     * @param {?} treeControl
     * @return {?}
     */
    expandFlattenedNodes(nodes, treeControl) {
        /** @type {?} */
        let results = [];
        /** @type {?} */
        let currentExpand = [];
        currentExpand[0] = true;
        nodes.forEach((/**
         * @param {?} node
         * @return {?}
         */
        node => {
            /** @type {?} */
            let expand = true;
            for (let i = 0; i <= this.getLevel(node); i++) {
                expand = expand && currentExpand[i];
            }
            if (expand) {
                results.push(node);
            }
            if (this.isExpandable(node)) {
                currentExpand[this.getLevel(node) + 1] = treeControl.isExpanded(node);
            }
        }));
        return results;
    }
}
/**
 * @template T, F
 */
class NtTreeFlatDataSource extends DataSource {
    /**
     * @param {?} treeControl
     * @param {?} treeFlattener
     * @param {?=} initialData
     */
    constructor(treeControl, treeFlattener, initialData = []) {
        super();
        this.treeControl = treeControl;
        this.treeFlattener = treeFlattener;
        this._flattenedData = new BehaviorSubject([]);
        this._expandedData = new BehaviorSubject([]);
        this._data = new BehaviorSubject(initialData);
    }
    /**
     * @return {?}
     */
    get data() { return this._data.value; }
    /**
     * @param {?} value
     * @return {?}
     */
    set data(value) {
        this._data.next(value);
        this._flattenedData.next(this.treeFlattener.flattenNodes(this.data));
        this.treeControl.dataNodes = this._flattenedData.value;
    }
    /**
     * @param {?} collectionViewer
     * @return {?}
     */
    connect(collectionViewer) {
        /** @type {?} */
        const changes = [
            collectionViewer.viewChange,
            (/** @type {?} */ (this.treeControl.expansionModel.onChange)),
            this._flattenedData
        ];
        return merge(...changes).pipe(map((/**
         * @return {?}
         */
        () => {
            this._expandedData.next(this.treeFlattener.expandFlattenedNodes(this._flattenedData.value, this.treeControl));
            return this._expandedData.value;
        })));
    }
    /**
     * @return {?}
     */
    disconnect() { }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtTreeNodeOutletDirective {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
}
NtTreeNodeOutletDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeOutlet]',
                providers: [
                    { provide: CdkTreeNodeOutlet, useExisting: NtTreeNodeOutletDirective }
                ]
            },] },
];
/** @nocollapse */
NtTreeNodeOutletDirective.ctorParameters = () => [
    { type: ViewContainerRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTreeNodeDirective extends CdkTreeNode {
    /**
     * @param {?} _elementRef
     * @param {?} _tree
     * @param {?} tabIndex
     */
    constructor(_elementRef, _tree, tabIndex) {
        super(_elementRef, _tree);
        this._elementRef = _elementRef;
        this._tree = _tree;
        this._disabled = false;
        this._tabIndex = 0;
        this.role = 'treeitem';
        this.tabIndex = Number(tabIndex) || 0;
    }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get tabIndex() { return this.disabled ? -1 : this._tabIndex; }
    /**
     * @param {?} value
     * @return {?}
     */
    set tabIndex(value) { this._tabIndex = value != null ? value : 0; }
}
NtTreeNodeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-tree-node, [nt-tree-node]',
                exportAs: 'ntTreeNode',
                inputs: ['disabled', 'tabIndex'],
                host: {
                    'class': 'nt-tree-node'
                },
                providers: [{ provide: CdkTreeNode, useExisting: NtTreeNodeDirective }]
            },] },
];
/** @nocollapse */
NtTreeNodeDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: CdkTree },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
];
NtTreeNodeDirective.propDecorators = {
    role: [{ type: Input }]
};
/**
 * @template T
 */
class NtTreeNodeDefDirective extends CdkTreeNodeDef {
}
NtTreeNodeDefDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeDef]',
                inputs: [
                    'when: ntTreeNodeDefWhen'
                ],
                providers: [{ provide: CdkTreeNodeDef, useExisting: NtTreeNodeDefDirective }]
            },] },
];
NtTreeNodeDefDirective.propDecorators = {
    data: [{ type: Input, args: ['ntTreeNode',] }]
};
/**
 * @template T
 */
class NtNestedTreeNodeDirective extends CdkNestedTreeNode {
    /**
     * @param {?} _elementRef
     * @param {?} _tree
     * @param {?} _differs
     * @param {?} tabIndex
     */
    constructor(_elementRef, _tree, _differs, tabIndex) {
        super(_elementRef, _tree, _differs);
        this._elementRef = _elementRef;
        this._tree = _tree;
        this._differs = _differs;
        this._disabled = false;
        this._tabIndex = 0;
        this.tabIndex = Number(tabIndex) || 0;
    }
    /**
     * @return {?}
     */
    get disabled() { return this._disabled; }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) { this._disabled = coerceBooleanProperty(value); }
    /**
     * @return {?}
     */
    get tabIndex() { return this.disabled ? -1 : this._tabIndex; }
    /**
     * @param {?} value
     * @return {?}
     */
    set tabIndex(value) { this._tabIndex = value != null ? value : 0; }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        super.ngAfterContentInit();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
NtNestedTreeNodeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'nt-nested-tree-node, [nt-nested-tree-node]',
                exportAs: 'ntNestedTreeNode',
                host: {
                    'class': 'nt-nested-tree-node',
                },
                inputs: ['disabled', 'tabIndex'],
                providers: [
                    { provide: CdkNestedTreeNode, useExisting: NtNestedTreeNodeDirective },
                    { provide: CdkTreeNode, useExisting: NtNestedTreeNodeDirective }
                ]
            },] },
];
/** @nocollapse */
NtNestedTreeNodeDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: CdkTree },
    { type: IterableDiffers },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
];
NtNestedTreeNodeDirective.propDecorators = {
    node: [{ type: Input, args: ['ntNestedTreeNode',] }],
    nodeOutlet: [{ type: ContentChildren, args: [NtTreeNodeOutletDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTreeNodePaddingDirective extends CdkTreeNodePadding {
}
NtTreeNodePaddingDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodePadding]',
                providers: [{ provide: CdkTreeNodePadding, useExisting: NtTreeNodePaddingDirective }]
            },] },
];
NtTreeNodePaddingDirective.propDecorators = {
    level: [{ type: Input, args: ['ntTreeNodePadding',] }],
    indent: [{ type: Input, args: ['ntTreeNodePaddingIndent',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTreeNodeToggleDirective extends CdkTreeNodeToggle {
    constructor() {
        super(...arguments);
        this.recursive = false;
    }
}
NtTreeNodeToggleDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ntTreeNodeToggle]',
                host: {
                    '(click)': '_toggle($event)',
                },
                providers: [{ provide: CdkTreeNodeToggle, useExisting: NtTreeNodeToggleDirective }]
            },] },
];
NtTreeNodeToggleDirective.propDecorators = {
    recursive: [{ type: Input, args: ['ntTreeNodeToggleRecursive',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtTreeComponent extends CdkTree {
}
NtTreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-tree, [nt-tree]',
                template: `<ng-container ntTreeNodeOutlet></ng-container>`,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [
                    { provide: CdkTree, useExisting: NtTreeComponent }
                ],
                host: {
                    'class': 'nt-tree'
                }
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const NT_TREE_DIRECTIVES = [
    NtNestedTreeNodeDirective,
    NtTreeNodeDefDirective,
    NtTreeNodePaddingDirective,
    NtTreeNodeToggleDirective,
    NtTreeComponent,
    NtTreeNodeDirective,
    NtTreeNodeOutletDirective
];
class NtTreeModule {
}
NtTreeModule.decorators = [
    { type: NgModule, args: [{
                imports: [CdkTreeModule, CommonModule],
                exports: NT_TREE_DIRECTIVES,
                declarations: NT_TREE_DIRECTIVES
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtTreeNestedDataSource, NtTreeFlattener, NtTreeFlatDataSource, NtTreeNodeDirective, NtTreeNodeDefDirective, NtNestedTreeNodeDirective, NtTreeNodeOutletDirective, NtTreeNodePaddingDirective, NtTreeNodeToggleDirective, NtTreeComponent, NtTreeModule };
//# sourceMappingURL=tree.js.map
