import { InjectionToken, Pipe, Component, EventEmitter, Inject, Input, Optional, Output, Self, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { transition, trigger } from '@angular/animations';
import { coerceArray, coerceBooleanProperty, coerceNumberProperty } from '@angular/cdk/coercion';
import { NgControl } from '@angular/forms';
import { fadeOut, NT_UPLOAD_HANDLER, NtUploadFile, NtUploadStatus } from '@ng-tangram/components/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';
import { NtProgressModule } from '@ng-tangram/components/progress';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DEFAULT_FILE_ICONS = {
    default: 'far fa-file-alt',
    archive: 'far fa-file-archive',
    code: 'far fa-file-code',
    image: 'far fa-file-image',
    audio: 'far fa-file-audio',
    video: 'far fa-file-video',
    pdf: 'far fa-file-pdf',
    word: 'far fa-file-word',
    excel: 'far fa-file-excel',
    ppt: 'far fa-file-powerpoint',
};
/** @type {?} */
var NT_FILE_ICONS = new InjectionToken('nt-file-icons');
/** @type {?} */
var NT_FILE_EXTENSIONS = {
    'rar': 'archive',
    'zip': 'archive',
    'cab': 'archive',
    'iso': 'archive',
    'jar': 'archive',
    '7z': 'archive',
    'tar': 'archive',
    'gz': 'archive',
    'bz2': 'archive',
    'z': 'archive',
    'jpg': 'image',
    'bmp': 'image',
    'eps': 'image',
    'gif': 'image',
    'mif': 'image',
    'miff': 'image',
    'png': 'image',
    'tif': 'image',
    'tiff': 'image',
    'svg': 'image',
    'wmf': 'image',
    'jpe': 'image',
    'jpeg': 'image',
    'dib': 'image',
    'ico': 'image',
    'tga': 'image',
    'cut': 'image',
    'pic': 'image',
    'wma': 'audio',
    'ra': 'audio',
    'ogg': 'audio',
    'mpc': 'audio',
    'm4a': 'audio',
    'aac': 'audio',
    'mpa': 'audio',
    'mp2': 'audio',
    'm1a': 'audio',
    'm2a': 'audio',
    'mp3': 'audio',
    'mid': 'audio',
    'midi': 'audio',
    'rmi': 'audio',
    'mka': 'audio',
    'ac3': 'audio',
    'dts': 'audio',
    'cda': 'audio',
    'au': 'audio',
    'snd': 'audio',
    'aif': 'audio',
    'aifc': 'audio',
    'aiff': 'audio',
    'mp4': 'video',
    'avi': 'video',
    'wav': 'video',
    'wmv': 'video',
    'rmvb': 'video',
    'rm': 'video',
    'flv': 'video',
    'mkv': 'video',
    'mpg': 'video',
    'mpeg': 'video',
    'mov': 'video',
    '3gp': 'video',
    'doc': 'word',
    'docx': 'word',
    'xls': 'excel',
    'xlsx': 'excel',
    'ppt': 'ppt',
    'pptx': 'ppt'
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var FILE_SIZE_UNITS = ['bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];
var NtFileSizePipe = /** @class */ (function () {
    function NtFileSizePipe() {
    }
    /**
     * @param {?=} bytes
     * @param {?=} precision
     * @return {?}
     */
    NtFileSizePipe.prototype.transform = /**
     * @param {?=} bytes
     * @param {?=} precision
     * @return {?}
     */
    function (bytes, precision) {
        if (bytes === void 0) { bytes = 0; }
        if (precision === void 0) { precision = 2; }
        if (isNaN(parseFloat(String(bytes))) || !isFinite(bytes)) {
            return '?';
        }
        /** @type {?} */
        var unit = 0;
        while (bytes >= 1024) {
            bytes /= 1024;
            unit++;
        }
        return bytes.toFixed(+precision) + ' ' + FILE_SIZE_UNITS[unit];
    };
    NtFileSizePipe.decorators = [
        { type: Pipe, args: [{ name: 'ntFileSize' },] },
    ];
    return NtFileSizePipe;
}());

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtFileAcceptError = /** @class */ (function () {
    function NtFileAcceptError(file, fileAccept, allowAccepts) {
        this.file = file;
        this.fileAccept = fileAccept;
        this.allowAccepts = allowAccepts;
    }
    return NtFileAcceptError;
}());
var NtFileSizeError = /** @class */ (function () {
    function NtFileSizeError(file, maxSize, maxSizeString) {
        this.file = file;
        this.maxSize = maxSize;
        this.maxSizeString = maxSizeString;
    }
    return NtFileSizeError;
}());
var NtFileUploadError = /** @class */ (function () {
    function NtFileUploadError(status, statusText) {
        this.status = status;
        this.statusText = statusText;
    }
    return NtFileUploadError;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var uniqueId = 0;
var NtFile = /** @class */ (function (_super) {
    __extends(NtFile, _super);
    function NtFile() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.id = "nt-file-" + uniqueId++;
        return _this;
    }
    return NtFile;
}(NtUploadFile));
var NtFileComponent = /** @class */ (function (_super) {
    __extends(NtFileComponent, _super);
    function NtFileComponent(ngControl, _uploader, icons) {
        var _this = _super.call(this) || this;
        _this.ngControl = ngControl;
        _this._uploader = _uploader;
        _this.icons = icons;
        _this._destroy = new Subject();
        _this.files = [];
        _this._value = [];
        _this._disabled = false;
        _this._required = false;
        _this._accept = ['*'];
        _this._maxSize = 5;
        _this._maxFiles = 1;
        _this._autoupload = true;
        _this.url = '';
        _this.name = '';
        _this.error = new EventEmitter();
        _this.remove = new EventEmitter();
        _this._onChange = (/**
         * @return {?}
         */
        function () { });
        _this._onTouched = (/**
         * @return {?}
         */
        function () { });
        if (_this.ngControl) {
            _this.ngControl.valueAccessor = _this;
        }
        return _this;
    }
    Object.defineProperty(NtFileComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () { return this._value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "required", {
        get: /**
         * @return {?}
         */
        function () { return this._required; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._required = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "accept", {
        get: /**
         * @return {?}
         */
        function () { return this._accept; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (typeof value === 'string') {
                this._accept = value.replace(' ', '').split(',');
            }
            else {
                this._accept = coerceArray(value);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "maxSize", {
        get: /**
         * @return {?}
         */
        function () { return this._maxSize; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._maxSize = coerceNumberProperty(value, 5); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "maxFiles", {
        get: /**
         * @return {?}
         */
        function () { return this._maxFiles; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._maxFiles = coerceNumberProperty(value, 1); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtFileComponent.prototype, "autoupload", {
        get: /**
         * @return {?}
         */
        function () { return this._autoupload; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._autoupload = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NtFileComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    /**
     * @return {?}
     */
    NtFileComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroy.next();
        this._destroy.complete();
        this.error.complete();
        this.remove.complete();
    };
    /**
     * @return {?}
     */
    NtFileComponent.prototype.onTriggerClick = /**
     * @return {?}
     */
    function () {
        if (this.files.length < this.maxFiles) {
            this.fileElement.nativeElement.click();
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NtFileComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        var _a, _b;
        if (value) {
            this._value.length = 0;
            (_a = this._value).push.apply(_a, value);
            this.files.length = 0;
            (_b = this.files).push.apply(_b, this._value);
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtFileComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtFileComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    /**
     * @return {?}
     */
    NtFileComponent.prototype._fileChanged = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var file = this.fileElement.nativeElement.files[0];
        if (file && this.files.length < this.maxFiles) {
            if (!this._fileSizeValid(file)) {
                this.error.next(new NtFileSizeError(file, this.maxSize * 1024 * 1024, this.maxSize + "MB"));
                return;
            }
            if (!this._fileTypeValid(file)) {
                this.error.next(new NtFileAcceptError(file, file.type));
                return;
            }
            /** @type {?} */
            var ntFile_1 = new NtFile(file.name, file.size, file.type);
            this.files.push(ntFile_1);
            /** @type {?} */
            var handlers = {
                begin: (/**
                 * @return {?}
                 */
                function () { return _this._onUploadBegin(ntFile_1); }),
                progress: (/**
                 * @param {?} percent
                 * @return {?}
                 */
                function (percent) { return _this._onUploadProgress(percent, ntFile_1); }),
                done: (/**
                 * @return {?}
                 */
                function () { return _this._onUploadDone(ntFile_1); })
            };
            if (this.autoupload) {
                ntFile_1.uploader = this._uploader
                    .upload(this.url, file, this.name, handlers)
                    .pipe(takeUntil(this._destroy))
                    .subscribe((/**
                 * @param {?} result
                 * @return {?}
                 */
                function (result) {
                    if ((ntFile_1.status = result.status) === NtUploadStatus.SUCCESS) {
                        ntFile_1.data = result.data;
                        _this._value.push(ntFile_1);
                        _this._onChange(_this._value);
                    }
                }), (/**
                 * @param {?} error
                 * @return {?}
                 */
                function (error) {
                    ntFile_1.status = NtUploadStatus.ERROR;
                    ntFile_1.error = error.statusText;
                    ntFile_1.progress = 100;
                    _this.error.next(new NtFileUploadError(error.status, error.statusText));
                }));
            }
        }
        this._onTouched && this._onTouched();
        this.fileElement.nativeElement.value = '';
    };
    /**
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype.removeFile = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        if (this.disabled) {
            return;
        }
        file.uploader && file.uploader.unsubscribe();
        /** @type {?} */
        var vindex = this._value.indexOf(file);
        vindex > -1 && this._value.splice(vindex, 1);
        /** @type {?} */
        var findex = this.files.indexOf(file);
        findex > -1 && this.files.splice(findex, 1);
        this.remove.next(file);
        this._onChange(this._value);
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    NtFileComponent.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this._disabled = isDisabled;
    };
    /**
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype.getFileIcon = /**
     * @param {?} file
     * @return {?}
     */
    function (file) {
        /** @type {?} */
        var extension = file.name.substring(file.name.lastIndexOf('.') + 1, file.name.length);
        /** @type {?} */
        var icon = NT_FILE_EXTENSIONS[extension] || 'default';
        return this.icons[icon];
    };
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype._fileTypeValid = /**
     * @private
     * @param {?} file
     * @return {?}
     */
    function (file) {
        if (file) {
            return this.accept.indexOf(file.type) > -1 || this.accept.indexOf('*') > -1;
        }
        return false;
    };
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype._fileSizeValid = /**
     * @private
     * @param {?} file
     * @return {?}
     */
    function (file) {
        if (file) {
            return file.size <= this.maxSize * 1024 * 1024;
        }
        return false;
    };
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype._onUploadBegin = /**
     * @private
     * @param {?} file
     * @return {?}
     */
    function (file) {
        file.status = NtUploadStatus.SENDING;
    };
    /**
     * @private
     * @param {?} percent
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype._onUploadProgress = /**
     * @private
     * @param {?} percent
     * @param {?} file
     * @return {?}
     */
    function (percent, file) {
        file.progress = percent;
    };
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    NtFileComponent.prototype._onUploadDone = /**
     * @private
     * @param {?} file
     * @return {?}
     */
    function (file) {
        file.status = NtUploadStatus.SUCCESS;
    };
    NtFileComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-file',
                    template: "<button class=\"button hollow nt-file-trigger\" type=\"button\" (click)=\"onTriggerClick()\" [disabled]=\"files.length >= maxFiles || disabled\"> <ng-content></ng-content> </button> <input #fileElement type=\"file\" [accept]=\"accept\" (change)=\"_fileChanged()\" hidden> <div class=\"nt-file-list\"> <div class=\"nt-file-item\" *ngFor=\"let file of files\" [@fadeOut]> <div class=\"nt-file-info\" [class.success]=\"file.status === 2\" [class.error]=\"file.status === 3\" [class.remote]=\"file.status === 4\"> <span class=\"nt-file-symbol {{ getFileIcon(file) }}\"></span>&nbsp; <a *ngIf=\"file.link\" [href]=\"file.link\" target=\"_blank\">{{file.name}}</a> <ng-container *ngIf=\"!file.link\">{{file.name}}</ng-container> &nbsp;{{ file.size | ntFileSize }} &nbsp;<span class=\"nt-file-error\" *ngIf=\"file.error\">Error&nbsp;-&nbsp;{{file.error}}</span> <div class=\"nt-close-button\" *ngIf=\"!disabled\" (click)=\"removeFile(file)\"><span aria-hidden=\"true\">&times;</span></div> </div> <nt-progress *ngIf=\"file.status === 1\" [@fadeOut] [class.success]=\"file.status === 2\" [class.alert]=\"file.status === 3\" [value]=\"file.progress\" size=\"tiny\"></nt-progress> </div> </div> ",
                    host: {
                        'class': 'nt-file'
                    },
                    encapsulation: ViewEncapsulation.None,
                    providers: [
                        { provide: NtFormFieldControl, useExisting: NtFileComponent }
                    ],
                    animations: [
                        trigger('fadeOut', [
                            transition('* => void', fadeOut(.3))
                        ])
                    ]
                },] },
    ];
    /** @nocollapse */
    NtFileComponent.ctorParameters = function () { return [
        { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
        { type: undefined, decorators: [{ type: Inject, args: [NT_UPLOAD_HANDLER,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [NT_FILE_ICONS,] }] }
    ]; };
    NtFileComponent.propDecorators = {
        disabled: [{ type: Input }],
        required: [{ type: Input }],
        accept: [{ type: Input }],
        maxSize: [{ type: Input }],
        maxFiles: [{ type: Input }],
        autoupload: [{ type: Input }],
        url: [{ type: Input }],
        name: [{ type: Input }],
        fileElement: [{ type: ViewChild, args: ['fileElement', { static: true },] }],
        error: [{ type: Output }],
        remove: [{ type: Output }]
    };
    return NtFileComponent;
}(NtFormFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ɵ0 = DEFAULT_FILE_ICONS;
var NtFileModule = /** @class */ (function () {
    function NtFileModule() {
    }
    /**
     * @param {?=} icons
     * @return {?}
     */
    NtFileModule.forRoot = /**
     * @param {?=} icons
     * @return {?}
     */
    function (icons) {
        if (icons === void 0) { icons = DEFAULT_FILE_ICONS; }
        return {
            ngModule: NtFileModule,
            providers: [
                { provide: NT_FILE_ICONS, useValue: icons }
            ]
        };
    };
    NtFileModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, NtProgressModule],
                    exports: [NtFileComponent, NtFileSizePipe],
                    declarations: [NtFileComponent, NtFileSizePipe],
                    providers: [
                        { provide: NT_FILE_ICONS, useValue: ɵ0 }
                    ]
                },] },
    ];
    return NtFileModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtFileModule, NtFile, NtFileComponent, NtFileSizePipe, NtFileAcceptError, NtFileSizeError, NtFileUploadError, DEFAULT_FILE_ICONS, NT_FILE_ICONS, NT_FILE_EXTENSIONS };
//# sourceMappingURL=file.js.map
