import { InjectionToken, ChangeDetectionStrategy, ChangeDetectorRef, Component, ContentChildren, ElementRef, EventEmitter, Inject, Input, isDevMode, NgZone, Optional, Output, Self, ViewChild, ViewEncapsulation, NgModule } from '@angular/core';
import { defer, merge, Subject } from 'rxjs';
import { filter, startWith, switchMap, take, takeUntil } from 'rxjs/operators';
import { transition, trigger } from '@angular/animations';
import { ActiveDescendantKeyManager } from '@angular/cdk/a11y';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { SelectionModel } from '@angular/cdk/collections';
import { DOWN_ARROW, END, ENTER, HOME, LEFT_ARROW, RIGHT_ARROW, SPACE, UP_ARROW } from '@angular/cdk/keycodes';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { NgControl } from '@angular/forms';
import { BOTTOM_LEFT, fadeIn, fadeOut, NT_OPTION_PARENT_COMPONENT, NtOptionComponent, NtOverlayComponent, TOP_LEFT, NtOptionModule, NtOverlayModule } from '@ng-tangram/components/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var NT_SELECT_ICONS = new InjectionToken('nt-select-icons');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function getNtSelectDynamicMultipleError() {
    return Error('Cannot change `multiple` mode of select after initialization.');
}
/**
 * @return {?}
 */
function getNtSelectNonArrayValueError() {
    return Error('Value must be an array in multiple-selection mode.');
}
/**
 * @return {?}
 */
function getNtSelectNonFunctionValueError() {
    return Error('`compareWith` must be a function.');
}
var NtSelectChange = /** @class */ (function () {
    function NtSelectChange(source, value) {
        this.source = source;
        this.value = value;
    }
    return NtSelectChange;
}());
var NtSelectComponent = /** @class */ (function (_super) {
    __extends(NtSelectComponent, _super);
    function NtSelectComponent(_ngZone, _elementRef, _changeDetectorRef, ngControl, icons) {
        var _this = _super.call(this) || this;
        _this._ngZone = _ngZone;
        _this._elementRef = _elementRef;
        _this._changeDetectorRef = _changeDetectorRef;
        _this.ngControl = ngControl;
        _this.icons = icons;
        _this._destroy = new Subject();
        _this._positionPairs = [BOTTOM_LEFT, TOP_LEFT];
        _this._compareWith = (/**
         * @param {?} o1
         * @param {?} o2
         * @return {?}
         */
        function (o1, o2) { return o1 === o2; });
        _this._onChange = (/**
         * @return {?}
         */
        function () { });
        _this._onTouched = (/**
         * @return {?}
         */
        function () { });
        _this._focused = false;
        _this._disabled = false;
        _this._required = false;
        _this._state = '';
        _this._multiple = false;
        _this.filterNotFound = 'Not Found';
        _this._placeholder = '';
        _this.afterOpen = new EventEmitter();
        _this.afterClosed = new EventEmitter();
        _this.beforeOpen = new EventEmitter();
        _this.beforeClosed = new EventEmitter();
        _this.positionChange = new EventEmitter();
        _this.selectionChange = new EventEmitter();
        _this.valueChange = new EventEmitter();
        _this.optionSelectionChanges = defer((/**
         * @return {?}
         */
        function () {
            if (_this.options) {
                return merge.apply(void 0, _this.options.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) { return option.selectionChange; })));
            }
            return _this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            function () { return _this.optionSelectionChanges; })));
        }));
        _this.origin = new CdkOverlayOrigin(_this._elementRef);
        if (_this.ngControl) {
            _this.ngControl.valueAccessor = _this;
        }
        return _this;
    }
    Object.defineProperty(NtSelectComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () { return this._value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "triggerValue", {
        get: /**
         * @return {?}
         */
        function () {
            if (this.empty) {
                return '';
            }
            if (this._multiple) {
                /** @type {?} */
                var selectedOptions = this._selectionModel.selected.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) { return option.label; }));
                return selectedOptions.join(', ');
            }
            return this._selectionModel.selected[0].label;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "empty", {
        get: /**
         * @return {?}
         */
        function () { return !this._selectionModel || this._selectionModel.isEmpty(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "optionEmpty", {
        get: /**
         * @return {?}
         */
        function () {
            return this.options ? this.options.filter((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return !option.hidden; })).length === 0 : false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "focused", {
        get: /**
         * @return {?}
         */
        function () { return this._focused; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "required", {
        get: /**
         * @return {?}
         */
        function () { return this._required; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._required = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "state", {
        get: /**
         * @return {?}
         */
        function () { return this._state; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "width", {
        get: /**
         * @return {?}
         */
        function () { return this._width; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "multiple", {
        get: /**
         * @return {?}
         */
        function () { return this._multiple; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._multiple = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "filter", {
        get: /**
         * @return {?}
         */
        function () { return this._filter; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (typeof value === 'function') {
                this._filter = value;
            }
            else if (coerceBooleanProperty(value)) {
                this._filter = (/**
                 * @param {?} keyword
                 * @param {?} option
                 * @return {?}
                 */
                function (keyword, option) {
                    /** @type {?} */
                    var regex = new RegExp(keyword.toLowerCase(), 'g');
                    return regex.test(option.label.toLowerCase());
                });
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "selected", {
        get: /**
         * @return {?}
         */
        function () {
            return this.multiple ? this._selectionModel.selected : this._selectionModel.selected[0];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "placeholder", {
        get: /**
         * @return {?}
         */
        function () {
            if (this.empty) {
                return this._placeholder;
            }
            return this.filter ? this.triggerValue : this._placeholder;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._placeholder = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtSelectComponent.prototype, "compareWith", {
        set: /**
         * @param {?} fn
         * @return {?}
         */
        function (fn) {
            if (typeof fn !== 'function') {
                throw getNtSelectNonFunctionValueError();
            }
            else {
                this._compareWith = fn;
                if (this._selectionModel) {
                    this._initializeSelection();
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this._selectionModel = new SelectionModel(this.multiple, undefined, false);
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._initKeyManager();
        this.options.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        function () {
            _this._resetOptions();
            _this._initializeSelection();
        }));
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroy.next();
        this._destroy.complete();
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.onResize = /**
     * @return {?}
     */
    function () {
        this._width = this.inputElement.nativeElement.clientWidth;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype.onSearch = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var _this = this;
        if (this.focused && this.filter && !this.disabled) {
            /** @type {?} */
            var target_1 = event.target;
            this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.hidden = !_this.filter(target_1.value, option); }));
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._beforeOpen = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._state = 'folded';
        this.onResize();
        this._scrollActiveOptionIntoView();
        this._highlightCorrectOption();
        this.beforeOpen.next(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._beforeClosed = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._state = 'closed';
        this._onTouched();
        this.beforeClosed.next(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._afterOpen = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.afterOpen.next(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._afterClosed = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._resetFilterResult();
        this.inputElement.nativeElement.blur();
        this.afterClosed.next(event);
    };
    /**
     * @param {?} change
     * @return {?}
     */
    NtSelectComponent.prototype._positionChange = /**
     * @param {?} change
     * @return {?}
     */
    function (change) {
        this.positionChange.next(change);
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.onFocus = /**
     * @return {?}
     */
    function () {
        if (!this.overlay.isOpen && !this._disabled) {
            this.overlay.show();
            this._focused = true;
        }
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.onBlur = /**
     * @return {?}
     */
    function () {
        this._focused = false;
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.focus = /**
     * @return {?}
     */
    function () {
        this.inputElement.nativeElement.focus();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NtSelectComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (this.options) {
            this._setSelectionByValue(value);
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtSelectComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtSelectComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    /**
     * @return {?}
     */
    NtSelectComponent.prototype.clear = /**
     * @return {?}
     */
    function () {
        if (!this.disabled) {
            this._clearSelection();
            this._value = null;
            this._onChange(this.value);
            this.valueChange.next(this.value);
        }
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    NtSelectComponent.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this._disabled = isDisabled;
    };
    /** Handles all keydown events on the select. */
    /**
     * Handles all keydown events on the select.
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._handleKeydown = /**
     * Handles all keydown events on the select.
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (!this.disabled) {
            this.overlay.isOpen ? this._handleOpenKeydown(event) : this._handleClosedKeydown(event);
        }
    };
    /** Handles keyboard events when the selected is open. */
    /**
     * Handles keyboard events when the selected is open.
     * @private
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._handleOpenKeydown = /**
     * Handles keyboard events when the selected is open.
     * @private
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var keyCode = event.keyCode;
        /** @type {?} */
        var isArrowKey = keyCode === DOWN_ARROW || keyCode === UP_ARROW;
        /** @type {?} */
        var manager = this._keyManager;
        if (keyCode === HOME || keyCode === END) {
            event.preventDefault();
            keyCode === HOME ? manager.setFirstItemActive() : manager.setLastItemActive();
        }
        else if (isArrowKey && event.altKey) {
            event.preventDefault();
            this.overlay.hide();
        }
        else if ((keyCode === ENTER || keyCode === SPACE) && manager.activeItem) {
            event.preventDefault();
            manager.activeItem.selectViaInteraction();
            if (keyCode === ENTER && this.multiple) {
                this.overlay.hide();
            }
        }
        else {
            /** @type {?} */
            var previouslyFocusedIndex = manager.activeItemIndex;
            manager.onKeydown(event);
            if (this._multiple && isArrowKey && event.shiftKey && manager.activeItem &&
                manager.activeItemIndex !== previouslyFocusedIndex) {
                manager.activeItem.selectViaInteraction();
            }
        }
    };
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    NtSelectComponent.prototype._handleClosedKeydown = /**
     * @private
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var keyCode = event.keyCode;
        /** @type {?} */
        var isArrowKey = keyCode === DOWN_ARROW || keyCode === UP_ARROW ||
            keyCode === LEFT_ARROW || keyCode === RIGHT_ARROW;
        /** @type {?} */
        var isOpenKey = keyCode === ENTER || keyCode === SPACE;
        // Open the select on ALT + arrow key to match the native <select>
        if (isOpenKey || ((this.multiple || event.altKey) && isArrowKey)) {
            event.preventDefault(); // prevents the page from scrolling down when pressing space
            this.overlay.show();
        }
        else if (!this.multiple) {
            this._keyManager.onKeydown(event);
        }
    };
    /**
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._resetOptions = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var changedOrDestroyed = merge(this.options.changes, this._destroy);
        this.optionSelectionChanges
            .pipe(takeUntil(changedOrDestroyed), filter((/**
         * @param {?} event
         * @return {?}
         */
        function (event) { return event.isUserInput; })))
            .subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this._onSelect(event.source);
            if (!_this.multiple && _this.overlay.isOpen) {
                _this.overlay.hide();
            }
        }));
        // Listen to changes in the internal state of the options and react accordingly.
        // Handles cases like the labels of the selected options changing.
        merge.apply(void 0, this.options.map((/**
         * @param {?} option
         * @return {?}
         */
        function (option) { return option.stateChanges; }))).pipe(takeUntil(changedOrDestroyed))
            .subscribe((/**
         * @return {?}
         */
        function () {
            _this._changeDetectorRef.markForCheck();
        }));
        this._resetFilterResult();
    };
    /** Sets up a key manager to listen to keyboard events on the overlay panel. */
    /**
     * Sets up a key manager to listen to keyboard events on the overlay panel.
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._initKeyManager = /**
     * Sets up a key manager to listen to keyboard events on the overlay panel.
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        this._keyManager = new ActiveDescendantKeyManager(this.options)
            .withTypeAhead()
            .withVerticalOrientation();
        this._keyManager.tabOut.pipe(takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        function () {
            // Restore focus to the trigger before closing. Ensures that the focus
            // position won't be lost if the user got focus into the overlay.
            _this.focus();
            _this.overlay.hide();
            // this.inputElement.nativeElement.blur();
        }));
        this._keyManager.change.pipe(takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        function () {
            if (_this.overlay.isOpen && _this.paneElement) {
                _this._scrollActiveOptionIntoView();
            }
            else if (!_this.overlay.isOpen && !_this.multiple && _this._keyManager.activeItem) {
                _this._keyManager.activeItem.selectViaInteraction();
            }
        }));
    };
    /**
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._resetFilterResult = /**
     * @private
     * @return {?}
     */
    function () {
        this.options.forEach((/**
         * @param {?} option
         * @return {?}
         */
        function (option) { return option.hidden = false; }));
    };
    /**
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._initializeSelection = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        Promise.resolve().then((/**
         * @return {?}
         */
        function () {
            _this._setSelectionByValue(_this.ngControl ? _this.ngControl.value : _this.value);
        }));
    };
    /**
     * @private
     * @param {?} value
     * @param {?=} isUserInput
     * @return {?}
     */
    NtSelectComponent.prototype._setSelectionByValue = /**
     * @private
     * @param {?} value
     * @param {?=} isUserInput
     * @return {?}
     */
    function (value, isUserInput) {
        var _this = this;
        if (isUserInput === void 0) { isUserInput = false; }
        if (this.multiple && value) {
            if (!Array.isArray(value)) {
                throw getNtSelectNonArrayValueError();
            }
            this._clearSelection();
            value.forEach((/**
             * @param {?} currentValue
             * @return {?}
             */
            function (currentValue) { return _this._selectValue(currentValue, isUserInput); }));
            this._sortValues();
        }
        else {
            this._clearSelection();
            /** @type {?} */
            var correspondingOption = this._selectValue(value, isUserInput);
            // Shift focus to the active item. Note that we shouldn't do this in multiple
            // mode, because we don't know what option the user interacted with last.
            if (correspondingOption) {
                this._keyManager.setActiveItem(this.options.toArray().indexOf(correspondingOption));
            }
        }
        this._changeDetectorRef.markForCheck();
    };
    /**
     * @private
     * @param {?} value
     * @param {?=} isUserInput
     * @return {?}
     */
    NtSelectComponent.prototype._selectValue = /**
     * @private
     * @param {?} value
     * @param {?=} isUserInput
     * @return {?}
     */
    function (value, isUserInput) {
        var _this = this;
        if (isUserInput === void 0) { isUserInput = false; }
        /** @type {?} */
        var correspondingOption = this.options.find((/**
         * @param {?} option
         * @return {?}
         */
        function (option) {
            try {
                return option.value != null && _this._compareWith(option.value, value);
            }
            catch (error) {
                if (isDevMode()) {
                    console.warn(error);
                }
                return false;
            }
        }));
        if (correspondingOption) {
            isUserInput ? correspondingOption.selectViaInteraction() : correspondingOption.select();
            this._selectionModel.select(correspondingOption);
        }
        return correspondingOption;
    };
    /**
     * @private
     * @param {?=} fallbackValue
     * @return {?}
     */
    NtSelectComponent.prototype._propagateChanges = /**
     * @private
     * @param {?=} fallbackValue
     * @return {?}
     */
    function (fallbackValue) {
        /** @type {?} */
        var valueToEmit = null;
        if (this.multiple) {
            valueToEmit = ((/** @type {?} */ (this.selected))).map((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.value; }));
        }
        else {
            valueToEmit = this.selected ? ((/** @type {?} */ (this.selected))).value : fallbackValue;
        }
        this._value = valueToEmit;
        this.valueChange.emit(valueToEmit);
        this._onChange(valueToEmit);
        this.selectionChange.emit(new NtSelectChange(this, valueToEmit));
        this._changeDetectorRef.markForCheck();
    };
    /**
     * @private
     * @param {?=} skip
     * @return {?}
     */
    NtSelectComponent.prototype._clearSelection = /**
     * @private
     * @param {?=} skip
     * @return {?}
     */
    function (skip) {
        this._selectionModel.clear();
        this.options.forEach((/**
         * @param {?} option
         * @return {?}
         */
        function (option) {
            if (option !== skip) {
                option.deselect();
            }
        }));
    };
    /**
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._sortValues = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.multiple) {
            this._selectionModel.clear();
            this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) {
                if (option.selected) {
                    _this._selectionModel.select(option);
                }
            }));
        }
    };
    /**
     * @private
     * @param {?} option
     * @return {?}
     */
    NtSelectComponent.prototype._onSelect = /**
     * @private
     * @param {?} option
     * @return {?}
     */
    function (option) {
        /** @type {?} */
        var wasSelected = this._selectionModel.isSelected(option);
        if (this.multiple) {
            this._selectionModel.toggle(option);
            wasSelected ? option.deselect() : option.select();
            this._keyManager.setActiveItem(this.options.toArray().indexOf(option));
            this._sortValues();
        }
        else {
            this._clearSelection(option.value == null ? undefined : option);
            if (option.value == null) {
                this._propagateChanges(option.value);
            }
            else {
                this._selectionModel.select(option);
            }
        }
        if (wasSelected !== this._selectionModel.isSelected(option)) {
            this._propagateChanges();
        }
    };
    /**
     * Highlights the selected item. If no option is selected, it will highlight
     * the first item instead.
     */
    /**
     * Highlights the selected item. If no option is selected, it will highlight
     * the first item instead.
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._highlightCorrectOption = /**
     * Highlights the selected item. If no option is selected, it will highlight
     * the first item instead.
     * @private
     * @return {?}
     */
    function () {
        if (this._keyManager) {
            if (this.empty) {
                this._keyManager.setFirstItemActive();
            }
            else {
                /** @type {?} */
                var selectIndex = this.options.toArray().indexOf(this._selectionModel.selected[0]);
                this._keyManager.setActiveItem(selectIndex);
            }
        }
    };
    /**
     * @private
     * @return {?}
     */
    NtSelectComponent.prototype._scrollActiveOptionIntoView = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var activeOption = this._keyManager.activeItem || this.options.first;
        if (!activeOption) {
            return;
        }
        /** @type {?} */
        var panelScrollTop = this.paneElement.nativeElement.scrollTop;
        /** @type {?} */
        var panelScrollBottom = panelScrollTop + this.paneElement.nativeElement.offsetHeight;
        if (panelScrollBottom === 0) {
            this.paneElement.nativeElement.scrollTop = activeOption.getOffsetTop();
        }
        else if (activeOption.getOffsetTop() < panelScrollTop) {
            // 当活动选项超出 panel 的顶部范围时，重新设置 panel 可视区域。
            this.paneElement.nativeElement.scrollTop = activeOption.getOffsetTop();
        }
        else if (activeOption.getOffsetTop() + activeOption.getOffsetHeight() >= panelScrollBottom) {
            // 当活动选项超出 panel 的底部范围时，重新设置 panel 可视区域。
            /** @type {?} */
            var scrollOffset = this.paneElement.nativeElement.offsetHeight - activeOption.getOffsetHeight();
            this.paneElement.nativeElement.scrollTop = activeOption.getOffsetTop() - scrollOffset;
        }
    };
    NtSelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-select',
                    template: "<input #inputElement type=\"text\" (focus)=\"onFocus()\" (blur)=\"onBlur()\" (window:resize)=\"onResize()\" (keyup)=\"onSearch($event)\" [readonly]=\"!filter\" [placeholder]=\"placeholder\" [disabled]=\"disabled\" [value]=\"filter && focused ? '' : triggerValue\"> <span class=\"nt-select-caret {{ icons.caret }}\" [@fade] *ngIf=\"!triggerValue || disabled\"></span> <span class=\"nt-select-clear {{ icons.clear }}\" [@fade] *ngIf=\"triggerValue && !disabled\" (click)=\"clear()\"></span> <nt-overlay [origin]=\"origin\" trigger=\"click\" [positionPairs]=\"_positionPairs\" (afterOpen)=\"_afterOpen($event)\" (afterClosed)=\"_afterClosed($event)\" (beforeOpen)=\"_beforeOpen($event)\" (beforeClosed)=\"_beforeClosed($event)\" (positionChange)=\"_positionChange($event)\" (keydownEvents)=\"_handleKeydown($event)\"> <div #paneElement class=\"nt-select-pane\" [class.folded]=\"state === 'folded'\" [style.width.px]=\"width\"> <ng-content select=\"nt-option\"></ng-content> <div class=\"filter-empty\" *ngIf=\"optionEmpty\" (click)=\"overlay.hide()\"> {{ filterNotFound }} </div> </div> </nt-overlay> ",
                    providers: [
                        { provide: NT_OPTION_PARENT_COMPONENT, useExisting: NtSelectComponent },
                        { provide: NtFormFieldControl, useExisting: NtSelectComponent }
                    ],
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    animations: [
                        trigger('fade', [
                            transition('* => void', fadeOut(.15)),
                            transition('void => *', fadeIn(.15))
                        ])
                    ],
                    host: {
                        'class': 'nt-select',
                        '(resize)': 'onResize()',
                        '[class.focus]': 'overlay.isOpen'
                    }
                },] },
    ];
    /** @nocollapse */
    NtSelectComponent.ctorParameters = function () { return [
        { type: NgZone },
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
        { type: undefined, decorators: [{ type: Inject, args: [NT_SELECT_ICONS,] }] }
    ]; };
    NtSelectComponent.propDecorators = {
        disabled: [{ type: Input }],
        required: [{ type: Input }],
        multiple: [{ type: Input }],
        filter: [{ type: Input }],
        filterNotFound: [{ type: Input }],
        placeholder: [{ type: Input }],
        inputElement: [{ type: ViewChild, args: ['inputElement', { static: true },] }],
        paneElement: [{ type: ViewChild, args: ['paneElement', { static: true },] }],
        overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }],
        options: [{ type: ContentChildren, args: [NtOptionComponent,] }],
        afterOpen: [{ type: Output }],
        afterClosed: [{ type: Output }],
        beforeOpen: [{ type: Output }],
        beforeClosed: [{ type: Output }],
        positionChange: [{ type: Output }],
        selectionChange: [{ type: Output }],
        valueChange: [{ type: Output }],
        compareWith: [{ type: Input }]
    };
    return NtSelectComponent;
}(NtFormFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DEFAULT_SELECT_ICONS = {
    caret: 'fa fa-chevron-down',
    clear: 'fa fa-times'
};
var ɵ0 = DEFAULT_SELECT_ICONS;
var NtSelectModule = /** @class */ (function () {
    function NtSelectModule() {
    }
    /**
     * @param {?=} icons
     * @return {?}
     */
    NtSelectModule.forRoot = /**
     * @param {?=} icons
     * @return {?}
     */
    function (icons) {
        if (icons === void 0) { icons = DEFAULT_SELECT_ICONS; }
        return {
            ngModule: NtSelectModule,
            providers: [
                { provide: NT_SELECT_ICONS, useValue: icons }
            ]
        };
    };
    NtSelectModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, NtOverlayModule, NtOptionModule],
                    exports: [NtSelectComponent, NtOptionModule],
                    declarations: [NtSelectComponent],
                    providers: [
                        { provide: NT_SELECT_ICONS, useValue: ɵ0 }
                    ]
                },] },
    ];
    return NtSelectModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtSelectModule, getNtSelectDynamicMultipleError, getNtSelectNonArrayValueError, getNtSelectNonFunctionValueError, NtSelectChange, NtSelectComponent, NT_SELECT_ICONS };
//# sourceMappingURL=select.js.map
