import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CdkCell, CdkCellDef, CdkColumnDef, CdkFooterCell, CdkFooterCellDef, CdkHeaderCell, CdkHeaderCellDef, CDK_ROW_TEMPLATE, CdkFooterRow, CdkFooterRowDef, CdkHeaderRow, CdkHeaderRowDef, CdkRow, CdkRowDef, CDK_TABLE_TEMPLATE, CdkTable, CdkTableModule, DataSource } from '@angular/cdk/table';
import { Directive, ElementRef, EventEmitter, Input, ChangeDetectionStrategy, Component, ViewEncapsulation, Attribute, ChangeDetectorRef, ContentChildren, IterableDiffers, NgZone, Optional, Output, Inject, NgModule } from '@angular/core';
import { defer, merge, Subject, BehaviorSubject, combineLatest, Subscription } from 'rxjs';
import { filter, startWith, switchMap, take, takeUntil, map } from 'rxjs/operators';
import { Directionality } from '@angular/cdk/bidi';
import { SelectionModel } from '@angular/cdk/collections';
import { Platform } from '@angular/cdk/platform';
import { DOCUMENT, CommonModule } from '@angular/common';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
var NtColumnSort = {
    ASC: 'asc',
    DESC: 'desc',
    NONE: '',
};
var NtColumnSortChange = /** @class */ (function () {
    function NtColumnSortChange(isUserInput, column, sort) {
        if (isUserInput === void 0) { isUserInput = false; }
        this.isUserInput = isUserInput;
        this.column = column;
        this.sort = sort;
    }
    return NtColumnSortChange;
}());
/**
 * table 列单元格定义指令
 * \@description ntColumnCellDef 将在 > 0.5.x 版本中弃用
 */
var NtCellDefDirective = /** @class */ (function (_super) {
    __extends(NtCellDefDirective, _super);
    function NtCellDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtCellDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntCellDef], [ntColumnCellDef]',
                    providers: [{ provide: CdkCellDef, useExisting: NtCellDefDirective }]
                },] },
    ];
    return NtCellDefDirective;
}(CdkCellDef));
/**
 * table 头部单元格定义指令
 * \@description ntColumnHeaderDef 将在 > 0.5.x 版本中弃用
 */
var NtHeaderCellDefDirective = /** @class */ (function (_super) {
    __extends(NtHeaderCellDefDirective, _super);
    function NtHeaderCellDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtHeaderCellDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntHeaderCellDef], [ntColumnHeaderDef]',
                    providers: [{ provide: CdkHeaderCellDef, useExisting: NtHeaderCellDefDirective }]
                },] },
    ];
    return NtHeaderCellDefDirective;
}(CdkHeaderCellDef));
/**
 * table 底部单元格定义指令
 * \@description ntColumnFooterDef 将在 > 0.5.x 版本中弃用
 */
var NtFooterCellDefDirective = /** @class */ (function (_super) {
    __extends(NtFooterCellDefDirective, _super);
    function NtFooterCellDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtFooterCellDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntFooterCellDef], [ntColumnFooterDef]',
                    providers: [{ provide: CdkFooterCellDef, useExisting: NtFooterCellDefDirective }]
                },] },
    ];
    return NtFooterCellDefDirective;
}(CdkFooterCellDef));
/**
 * table 列定义指令
 */
var NtColumnDirective = /** @class */ (function (_super) {
    __extends(NtColumnDirective, _super);
    function NtColumnDirective() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._sortable = false;
        _this.sort = NtColumnSort.NONE;
        _this._sortChange = new EventEmitter();
        return _this;
    }
    Object.defineProperty(NtColumnDirective.prototype, "ntColumn", {
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this.name = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtColumnDirective.prototype, "sortable", {
        get: /**
         * @return {?}
         */
        function () { return this._sortable; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._sortable = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    /** 排序操作 */
    /**
     * 排序操作
     * @param {?=} isUserInput
     * @return {?}
     */
    NtColumnDirective.prototype.sorting = /**
     * 排序操作
     * @param {?=} isUserInput
     * @return {?}
     */
    function (isUserInput) {
        if (isUserInput === void 0) { isUserInput = false; }
        if (this.sortable) {
            /** 按照 升 -> 降 -> 无 的循环改变排序 */
            switch (this.sort) {
                case NtColumnSort.ASC:
                    this.sort = NtColumnSort.DESC;
                    break;
                case NtColumnSort.DESC:
                    this.sort = NtColumnSort.NONE;
                    break;
                default:
                    this.sort = NtColumnSort.ASC;
                    break;
            }
            this._sortChange.emit(new NtColumnSortChange(isUserInput, this.name, this.sort));
        }
    };
    NtColumnDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-column, [nt-column]',
                    providers: [{ provide: CdkColumnDef, useExisting: NtColumnDirective }],
                },] },
    ];
    NtColumnDirective.propDecorators = {
        ntColumn: [{ type: Input, args: ['nt-column',] }],
        name: [{ type: Input }],
        sticky: [{ type: Input }],
        stickyEnd: [{ type: Input }],
        sortable: [{ type: Input }]
    };
    return NtColumnDirective;
}(CdkColumnDef));
/**
 * table 头部单元格结构指令
 * \@description nt-column-header 将在 > 0.5.x 版本中弃用
 *
 */
var NtHeaderCellDirective = /** @class */ (function (_super) {
    __extends(NtHeaderCellDirective, _super);
    function NtHeaderCellDirective(columnDef, elementRef) {
        var _this = _super.call(this, columnDef, elementRef) || this;
        _this.columnDef = columnDef;
        elementRef.nativeElement.classList.add("nt-column-" + columnDef.cssClassFriendlyName);
        return _this;
    }
    NtHeaderCellDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-header-cell, nt-column-header, th[nt-header-cell], th[nt-column-header]',
                    host: {
                        'class': 'nt-header-cell',
                        '[class.nt-column-sortable]': 'columnDef.sortable',
                        '[class.asc]': 'columnDef.sort === "asc"',
                        '[class.desc]': 'columnDef.sort === "desc"',
                        '(click)': 'columnDef.sorting(true)',
                        'role': 'columnheader',
                    },
                },] },
    ];
    /** @nocollapse */
    NtHeaderCellDirective.ctorParameters = function () { return [
        { type: CdkColumnDef },
        { type: ElementRef }
    ]; };
    return NtHeaderCellDirective;
}(CdkHeaderCell));
/**
 * table 底部单元格结构指令
 * \@description nt-column-footer 将在 > 0.5.x 版本中弃用
 *
 */
var NtFooterCellDirective = /** @class */ (function (_super) {
    __extends(NtFooterCellDirective, _super);
    function NtFooterCellDirective(columnDef, elementRef) {
        var _this = _super.call(this, columnDef, elementRef) || this;
        elementRef.nativeElement.classList.add("nt-column-" + columnDef.cssClassFriendlyName);
        return _this;
    }
    NtFooterCellDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-footer-cell, nt-column-footer, td[nt-footer-cell], td[nt-column-footer]',
                    host: {
                        'class': 'nt-footer-cell',
                        'role': 'gridcell',
                    },
                },] },
    ];
    /** @nocollapse */
    NtFooterCellDirective.ctorParameters = function () { return [
        { type: CdkColumnDef },
        { type: ElementRef }
    ]; };
    return NtFooterCellDirective;
}(CdkFooterCell));
/**
 * table 单元格结构指令
 * \@description nt-column-cell 将在 > 0.5.x 版本中弃用
 */
var NtCellDirective = /** @class */ (function (_super) {
    __extends(NtCellDirective, _super);
    function NtCellDirective(columnDef, elementRef) {
        var _this = _super.call(this, columnDef, elementRef) || this;
        elementRef.nativeElement.classList.add("nt-column-" + columnDef.cssClassFriendlyName);
        return _this;
    }
    NtCellDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-cell, nt-column-cell, td[nt-cell], td[nt-column-cell]',
                    host: {
                        'class': 'nt-cell',
                        'role': 'gridcell',
                    },
                },] },
    ];
    /** @nocollapse */
    NtCellDirective.ctorParameters = function () { return [
        { type: CdkColumnDef },
        { type: ElementRef }
    ]; };
    return NtCellDirective;
}(CdkCell));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtHeaderRowDefDirective = /** @class */ (function (_super) {
    __extends(NtHeaderRowDefDirective, _super);
    function NtHeaderRowDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtHeaderRowDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntHeaderRowDef]',
                    providers: [{ provide: CdkHeaderRowDef, useExisting: NtHeaderRowDefDirective }],
                    inputs: ['columns: ntHeaderRowDef', 'sticky: ntHeaderRowDefSticky'],
                },] },
    ];
    return NtHeaderRowDefDirective;
}(CdkHeaderRowDef));
var NtFooterRowDefDirective = /** @class */ (function (_super) {
    __extends(NtFooterRowDefDirective, _super);
    function NtFooterRowDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtFooterRowDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntFooterRowDef]',
                    providers: [{ provide: CdkFooterRowDef, useExisting: NtFooterRowDefDirective }],
                    inputs: ['columns: ntFooterRowDef', 'sticky: ntFooterRowDefSticky'],
                },] },
    ];
    return NtFooterRowDefDirective;
}(CdkFooterRowDef));
/**
 * @template T
 */
var NtRowDefDirective = /** @class */ (function (_super) {
    __extends(NtRowDefDirective, _super);
    function NtRowDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtRowDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntRowDef]',
                    providers: [{ provide: CdkRowDef, useExisting: NtRowDefDirective }],
                    inputs: ['columns: ntRowDefColumns', 'when: ntRowDefWhen'],
                },] },
    ];
    return NtRowDefDirective;
}(CdkRowDef));
var NtHeaderRowComponent = /** @class */ (function (_super) {
    __extends(NtHeaderRowComponent, _super);
    function NtHeaderRowComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtHeaderRowComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-header-row, tr[nt-header-row]',
                    template: CDK_ROW_TEMPLATE,
                    host: {
                        'class': 'nt-header-row',
                        'role': 'row',
                    },
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    encapsulation: ViewEncapsulation.None,
                    exportAs: 'ntHeaderRow',
                    providers: [{ provide: CdkHeaderRow, useExisting: NtHeaderRowComponent }],
                },] },
    ];
    return NtHeaderRowComponent;
}(CdkHeaderRow));
var NtFooterRowComponent = /** @class */ (function (_super) {
    __extends(NtFooterRowComponent, _super);
    function NtFooterRowComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtFooterRowComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-footer-row, tr[nt-footer-row]',
                    template: CDK_ROW_TEMPLATE,
                    host: {
                        'class': 'nt-footer-row',
                        'role': 'row',
                    },
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    encapsulation: ViewEncapsulation.None,
                    exportAs: 'ntFooterRow',
                    providers: [{ provide: CdkFooterRow, useExisting: NtFooterRowComponent }],
                },] },
    ];
    return NtFooterRowComponent;
}(CdkFooterRow));
var NtRowComponent = /** @class */ (function (_super) {
    __extends(NtRowComponent, _super);
    function NtRowComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtRowComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-row, tr[nt-row]',
                    template: CDK_ROW_TEMPLATE,
                    host: {
                        'class': 'nt-row',
                        'role': 'row',
                    },
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    encapsulation: ViewEncapsulation.None,
                    exportAs: 'ntRow',
                    providers: [{ provide: CdkRow, useExisting: NtRowComponent }],
                },] },
    ];
    return NtRowComponent;
}(CdkRow));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var NtTableComponent = /** @class */ (function (_super) {
    __extends(NtTableComponent, _super);
    function NtTableComponent(_ngZone, _differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform) {
        var _this = _super.call(this, _differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform) || this;
        _this._ngZone = _ngZone;
        _this._differs = _differs;
        _this._changeDetectorRef = _changeDetectorRef;
        _this._elementRef = _elementRef;
        _this._dir = _dir;
        _this._selectionModel = new SelectionModel(true, undefined, false);
        _this._multiSortable = false;
        _this._selectable = false;
        _this._destroy = new Subject();
        _this.columSortChanges = defer((/**
         * @return {?}
         */
        function () {
            if (_this._contentColumns) {
                return merge.apply(void 0, _this._contentColumns.map((/**
                 * @param {?} column
                 * @return {?}
                 */
                function (column) { return column._sortChange; })));
            }
            return _this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            function () { return _this.columSortChanges; })));
        }));
        _this.selectedChange = new EventEmitter();
        _this.sortChange = new EventEmitter();
        return _this;
    }
    Object.defineProperty(NtTableComponent.prototype, "selectable", {
        get: /**
         * @return {?}
         */
        function () { return this._selectable; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._selectable = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtTableComponent.prototype, "isAllSelected", {
        get: /**
         * @return {?}
         */
        function () { return false; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtTableComponent.prototype, "selected", {
        get: /**
         * @return {?}
         */
        function () { return this._selectionModel.selected; },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NtTableComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._contentColumns.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        function () {
            _this._resetOptions();
            _this._changeDetectorRef.markForCheck();
        }));
    };
    /**
     * @return {?}
     */
    NtTableComponent.prototype.selectAll = /**
     * @return {?}
     */
    function () {
        if (!this.isAllSelected) {
            // this._selectionModel.select(...this.dataSource);
        }
        else {
            this._selectionModel.clear();
        }
        this.selectedChange.emit(this._selectionModel.selected);
    };
    /**
     * @param {?} item
     * @return {?}
     */
    NtTableComponent.prototype.select = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        /** @type {?} */
        var wasSelected = this.checkSelected(item);
        if (wasSelected) {
            this._selectionModel.deselect(item);
        }
        else {
            this._selectionModel.select(item);
        }
        this.selectedChange.emit(this._selectionModel.selected);
    };
    /**
     * @param {?} change
     * @return {?}
     */
    NtTableComponent.prototype.ngOnChanges = /**
     * @param {?} change
     * @return {?}
     */
    function (change) {
        if (change && change.dataSource && !change.dataSource.firstChange) {
            this._selectionModel.clear();
        }
    };
    /**
     * @param {?} item
     * @return {?}
     */
    NtTableComponent.prototype.checkSelected = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        return this._selectionModel.isSelected(item);
    };
    /**
     * @private
     * @param {?=} filter
     * @return {?}
     */
    NtTableComponent.prototype._clearSort = /**
     * @private
     * @param {?=} filter
     * @return {?}
     */
    function (filter$$1) {
        this._contentColumns
            .filter((/**
         * @param {?} column
         * @return {?}
         */
        function (column) { return column.name !== (filter$$1 ? filter$$1.column : ''); }))
            .forEach((/**
         * @param {?} column
         * @return {?}
         */
        function (column) { return column.sort = NtColumnSort.NONE; }));
        this._changeDetectorRef.markForCheck();
    };
    /**
     * @private
     * @return {?}
     */
    NtTableComponent.prototype._resetOptions = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var changedOrDestroyed = merge(this._contentColumns.changes, this._destroy);
        this.columSortChanges
            .pipe(takeUntil(changedOrDestroyed), filter((/**
         * @param {?} event
         * @return {?}
         */
        function (event) { return event.isUserInput; })))
            .subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (!_this._multiSortable) {
                _this._clearSort(event);
            }
            _this.sortChange.emit(event);
        }));
        merge.apply(void 0, this._contentColumns.map((/**
         * @param {?} column
         * @return {?}
         */
        function (column) { return column._sortChange; }))).pipe(takeUntil(changedOrDestroyed))
            .subscribe((/**
         * @return {?}
         */
        function () {
            _this._changeDetectorRef.markForCheck();
        }));
    };
    /**
     * @return {?}
     */
    NtTableComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroy.next();
        this._destroy.complete();
        _super.prototype.ngOnDestroy.call(this);
    };
    NtTableComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-table, table[nt-table]',
                    template: CDK_TABLE_TEMPLATE,
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: {
                        'class': 'nt-table'
                    }
                },] },
    ];
    /** @nocollapse */
    NtTableComponent.ctorParameters = function () { return [
        { type: NgZone },
        { type: IterableDiffers },
        { type: ChangeDetectorRef },
        { type: ElementRef },
        { type: String, decorators: [{ type: Attribute, args: ['role',] }] },
        { type: Directionality, decorators: [{ type: Optional }] },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: Platform }
    ]; };
    NtTableComponent.propDecorators = {
        selectable: [{ type: Input }],
        _contentColumns: [{ type: ContentChildren, args: [NtColumnDirective,] }],
        selectedChange: [{ type: Output }],
        sortChange: [{ type: Output }]
    };
    return NtTableComponent;
}(CdkTable));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtTableResizable = /** @class */ (function () {
    function NtTableResizable() {
    }
    NtTableResizable.decorators = [
        { type: Directive, args: [{
                    selector: 'table[ntTableResizable], nt-table[ntTableResizable]'
                },] },
    ];
    /** @nocollapse */
    NtTableResizable.ctorParameters = function () { return []; };
    return NtTableResizable;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var NT_CELL_DECLARATIONS = [
    NtCellDefDirective,
    NtHeaderCellDefDirective,
    NtFooterCellDefDirective,
    NtColumnDirective,
    NtHeaderCellDirective,
    NtFooterCellDirective,
    NtCellDirective
];
/** @type {?} */
var NT_ROW_DECLARATIONS = [
    NtHeaderRowDefDirective,
    NtFooterRowDefDirective,
    NtRowDefDirective,
    NtHeaderRowComponent,
    NtFooterRowComponent,
    NtRowComponent
];
var NtTableModule = /** @class */ (function () {
    function NtTableModule() {
    }
    NtTableModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, CdkTableModule],
                    exports: [
                        NtTableComponent,
                        NtTableResizable
                    ].concat(NT_CELL_DECLARATIONS, NT_ROW_DECLARATIONS),
                    declarations: [
                        NtTableComponent,
                        NtTableResizable
                    ].concat(NT_CELL_DECLARATIONS, NT_ROW_DECLARATIONS)
                },] },
    ];
    return NtTableModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
NtTableDataSource = /** @class */ (function (_super) {
    __extends(NtTableDataSource, _super);
    function NtTableDataSource(initialData) {
        if (initialData === void 0) { initialData = []; }
        var _this = _super.call(this) || this;
        /**
         * Stream emitting render data to the table (depends on ordered data changes).
         */
        _this._renderData = new BehaviorSubject([]);
        /**
         * Stream that emits when a new filter string is set on the data source.
         */
        _this._filter = new BehaviorSubject('');
        /**
         * Subscription to the changes that should trigger an update to the table's rendered rows, such
         * as filtering, sorting, pagination, or base data changes.
         */
        _this._renderChangesSubscription = Subscription.EMPTY;
        /**
         * Checks if a data object matches the data source's filter string. By default, each data object
         * is converted to a string of its properties and returns true if the filter has
         * at least one occurrence in that string. By default, the filter string has its whitespace
         * trimmed and the match is case-insensitive. May be overridden for a custom implementation of
         * filter matching.
         * @param data Data object used to check against the filter.
         * @param filter Filter string that has been set on the data source.
         * @return Whether the filter matches against the data
         */
        _this.filterPredicate = (/**
         * @param {?} data
         * @param {?} filter
         * @return {?}
         */
        function (data, filter$$1) {
            // Transform the data into a lowercase string of all property values.
            /** @type {?} */
            var accumulator = (/**
             * @param {?} currentTerm
             * @param {?} key
             * @return {?}
             */
            function (currentTerm, key) { return currentTerm + data[key]; });
            /** @type {?} */
            var dataStr = Object.keys(data).reduce(accumulator, '').toLowerCase();
            // Transform the filter by converting it to lowercase and removing whitespace.
            /** @type {?} */
            var transformedFilter = filter$$1.trim().toLowerCase();
            return dataStr.indexOf(transformedFilter) !== -1;
        });
        _this._data = new BehaviorSubject(initialData);
        _this._updateChangeSubscription();
        return _this;
    }
    Object.defineProperty(NtTableDataSource.prototype, "data", {
        /** Array of data that should be rendered by the table, where each object represents one row. */
        get: /**
         * Array of data that should be rendered by the table, where each object represents one row.
         * @return {?}
         */
        function () { return this._data.value; },
        set: /**
         * @param {?} data
         * @return {?}
         */
        function (data) { this._data.next(data); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtTableDataSource.prototype, "filter", {
        /**
         * Filter term that should be used to filter out objects from the data array. To override how
         * data objects match to this filter string, provide a custom function for filterPredicate.
         */
        get: /**
         * Filter term that should be used to filter out objects from the data array. To override how
         * data objects match to this filter string, provide a custom function for filterPredicate.
         * @return {?}
         */
        function () { return this._filter.value; },
        set: /**
         * @param {?} filter
         * @return {?}
         */
        function (filter$$1) { this._filter.next(filter$$1); },
        enumerable: true,
        configurable: true
    });
    /**
     * Subscribe to changes that should trigger an update to the table's rendered rows. When the
     * changes occur, process the current state of the filter, sort, and pagination along with
     * the provided base data and send it to the table for rendering.
     */
    /**
     * Subscribe to changes that should trigger an update to the table's rendered rows. When the
     * changes occur, process the current state of the filter, sort, and pagination along with
     * the provided base data and send it to the table for rendering.
     * @return {?}
     */
    NtTableDataSource.prototype._updateChangeSubscription = /**
     * Subscribe to changes that should trigger an update to the table's rendered rows. When the
     * changes occur, process the current state of the filter, sort, and pagination along with
     * the provided base data and send it to the table for rendering.
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var dataStream = this._data;
        // Watch for base data or filter changes to provide a filtered set of data.
        /** @type {?} */
        var filteredData = combineLatest(dataStream, this._filter)
            .pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        function (_a) {
            var data = _a[0];
            return _this._filterData(data);
        })));
        // Watched for paged data changes and send the result to the table to render.
        this._renderChangesSubscription.unsubscribe();
        this._renderChangesSubscription = filteredData.subscribe((/**
         * @param {?} data
         * @return {?}
         */
        function (data) { return _this._renderData.next(data); }));
    };
    /**
     * Returns a filtered data array where each filter object contains the filter string within
     * the result of the filterTermAccessor function. If no filter is set, returns the data array
     * as provided.
     */
    /**
     * Returns a filtered data array where each filter object contains the filter string within
     * the result of the filterTermAccessor function. If no filter is set, returns the data array
     * as provided.
     * @param {?} data
     * @return {?}
     */
    NtTableDataSource.prototype._filterData = /**
     * Returns a filtered data array where each filter object contains the filter string within
     * the result of the filterTermAccessor function. If no filter is set, returns the data array
     * as provided.
     * @param {?} data
     * @return {?}
     */
    function (data) {
        var _this = this;
        // If there is a filter string, filter out data that does not contain it.
        // Each data object is converted to a string using the function defined by filterTermAccessor.
        // May be overridden for customization.
        this.filteredData =
            !this.filter ? data : data.filter((/**
             * @param {?} obj
             * @return {?}
             */
            function (obj) { return _this.filterPredicate(obj, _this.filter); }));
        return this.filteredData;
    };
    /**
     * Used by the NtTable. Called when it connects to the data source.
     * @docs-private
     */
    /**
     * Used by the NtTable. Called when it connects to the data source.
     * \@docs-private
     * @return {?}
     */
    NtTableDataSource.prototype.connect = /**
     * Used by the NtTable. Called when it connects to the data source.
     * \@docs-private
     * @return {?}
     */
    function () { return this._renderData; };
    /**
     * Used by the NtTable. Called when it is destroyed. No-op.
     * @docs-private
     */
    /**
     * Used by the NtTable. Called when it is destroyed. No-op.
     * \@docs-private
     * @return {?}
     */
    NtTableDataSource.prototype.disconnect = /**
     * Used by the NtTable. Called when it is destroyed. No-op.
     * \@docs-private
     * @return {?}
     */
    function () { };
    return NtTableDataSource;
}(DataSource));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtTableModule, NtTableComponent, NtTableDataSource, NtColumnSort, NtColumnSortChange, NtCellDefDirective, NtHeaderCellDefDirective, NtFooterCellDefDirective, NtColumnDirective, NtHeaderCellDirective, NtFooterCellDirective, NtCellDirective, NtHeaderRowDefDirective, NtFooterRowDefDirective, NtRowDefDirective, NtHeaderRowComponent, NtFooterRowComponent, NtRowComponent, NtTableResizable as ɵa };
//# sourceMappingURL=table.js.map
