import { BehaviorSubject, merge } from 'rxjs';
import { map, take } from 'rxjs/operators';
import { DataSource } from '@angular/cdk/collections';
import { CdkTreeNodeOutlet, CdkNestedTreeNode, CdkTree, CdkTreeNode, CdkTreeNodeDef, CdkTreeNodePadding, CdkTreeNodeToggle, CdkTreeModule } from '@angular/cdk/tree';
import { Directive, ViewContainerRef, Attribute, ContentChildren, ElementRef, Input, IterableDiffers, ChangeDetectionStrategy, Component, ViewEncapsulation, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { CommonModule } from '@angular/common';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
NtTreeNestedDataSource = /** @class */ (function (_super) {
    __extends(NtTreeNestedDataSource, _super);
    function NtTreeNestedDataSource() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._data = new BehaviorSubject([]);
        return _this;
    }
    Object.defineProperty(NtTreeNestedDataSource.prototype, "data", {
        get: /**
         * @return {?}
         */
        function () { return this._data.value; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._data.next(value); },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} collectionViewer
     * @return {?}
     */
    NtTreeNestedDataSource.prototype.connect = /**
     * @param {?} collectionViewer
     * @return {?}
     */
    function (collectionViewer) {
        var _this = this;
        return merge.apply(void 0, [collectionViewer.viewChange, this._data]).pipe(map((/**
         * @return {?}
         */
        function () {
            return _this.data;
        })));
    };
    /**
     * @return {?}
     */
    NtTreeNestedDataSource.prototype.disconnect = /**
     * @return {?}
     */
    function () {
        // no op
    };
    return NtTreeNestedDataSource;
}(DataSource));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T, F
 */
var  /**
 * @template T, F
 */
NtTreeFlattener = /** @class */ (function () {
    function NtTreeFlattener(transformFunction, getLevel, isExpandable, getChildren) {
        this.transformFunction = transformFunction;
        this.getLevel = getLevel;
        this.isExpandable = isExpandable;
        this.getChildren = getChildren;
    }
    /**
     * @param {?} node
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    NtTreeFlattener.prototype._flattenNode = /**
     * @param {?} node
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    function (node, level, resultNodes, parentMap) {
        var _this = this;
        /** @type {?} */
        var flatNode = this.transformFunction(node, level);
        resultNodes.push(flatNode);
        if (this.isExpandable(flatNode)) {
            /** @type {?} */
            var childrenNodes = this.getChildren(node);
            if (Array.isArray(childrenNodes)) {
                this._flattenChildren(childrenNodes, level, resultNodes, parentMap);
            }
            else {
                childrenNodes.pipe(take(1)).subscribe((/**
                 * @param {?} children
                 * @return {?}
                 */
                function (children) {
                    _this._flattenChildren(children, level, resultNodes, parentMap);
                }));
            }
        }
        return resultNodes;
    };
    /**
     * @param {?} children
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    NtTreeFlattener.prototype._flattenChildren = /**
     * @param {?} children
     * @param {?} level
     * @param {?} resultNodes
     * @param {?} parentMap
     * @return {?}
     */
    function (children, level, resultNodes, parentMap) {
        var _this = this;
        children.forEach((/**
         * @param {?} child
         * @param {?} index
         * @return {?}
         */
        function (child, index) {
            /** @type {?} */
            var childParentMap = parentMap.slice();
            childParentMap.push(index !== children.length - 1);
            _this._flattenNode(child, level + 1, resultNodes, childParentMap);
        }));
    };
    /**
     * @param {?} structuredData
     * @return {?}
     */
    NtTreeFlattener.prototype.flattenNodes = /**
     * @param {?} structuredData
     * @return {?}
     */
    function (structuredData) {
        var _this = this;
        /** @type {?} */
        var resultNodes = [];
        structuredData.forEach((/**
         * @param {?} node
         * @return {?}
         */
        function (node) { return _this._flattenNode(node, 0, resultNodes, []); }));
        return resultNodes;
    };
    /**
     * @param {?} nodes
     * @param {?} treeControl
     * @return {?}
     */
    NtTreeFlattener.prototype.expandFlattenedNodes = /**
     * @param {?} nodes
     * @param {?} treeControl
     * @return {?}
     */
    function (nodes, treeControl) {
        var _this = this;
        /** @type {?} */
        var results = [];
        /** @type {?} */
        var currentExpand = [];
        currentExpand[0] = true;
        nodes.forEach((/**
         * @param {?} node
         * @return {?}
         */
        function (node) {
            /** @type {?} */
            var expand = true;
            for (var i = 0; i <= _this.getLevel(node); i++) {
                expand = expand && currentExpand[i];
            }
            if (expand) {
                results.push(node);
            }
            if (_this.isExpandable(node)) {
                currentExpand[_this.getLevel(node) + 1] = treeControl.isExpanded(node);
            }
        }));
        return results;
    };
    return NtTreeFlattener;
}());
/**
 * @template T, F
 */
var  /**
 * @template T, F
 */
NtTreeFlatDataSource = /** @class */ (function (_super) {
    __extends(NtTreeFlatDataSource, _super);
    function NtTreeFlatDataSource(treeControl, treeFlattener, initialData) {
        if (initialData === void 0) { initialData = []; }
        var _this = _super.call(this) || this;
        _this.treeControl = treeControl;
        _this.treeFlattener = treeFlattener;
        _this._flattenedData = new BehaviorSubject([]);
        _this._expandedData = new BehaviorSubject([]);
        _this._data = new BehaviorSubject(initialData);
        return _this;
    }
    Object.defineProperty(NtTreeFlatDataSource.prototype, "data", {
        get: /**
         * @return {?}
         */
        function () { return this._data.value; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._data.next(value);
            this._flattenedData.next(this.treeFlattener.flattenNodes(this.data));
            this.treeControl.dataNodes = this._flattenedData.value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} collectionViewer
     * @return {?}
     */
    NtTreeFlatDataSource.prototype.connect = /**
     * @param {?} collectionViewer
     * @return {?}
     */
    function (collectionViewer) {
        var _this = this;
        /** @type {?} */
        var changes = [
            collectionViewer.viewChange,
            (/** @type {?} */ (this.treeControl.expansionModel.onChange)),
            this._flattenedData
        ];
        return merge.apply(void 0, changes).pipe(map((/**
         * @return {?}
         */
        function () {
            _this._expandedData.next(_this.treeFlattener.expandFlattenedNodes(_this._flattenedData.value, _this.treeControl));
            return _this._expandedData.value;
        })));
    };
    /**
     * @return {?}
     */
    NtTreeFlatDataSource.prototype.disconnect = /**
     * @return {?}
     */
    function () { };
    return NtTreeFlatDataSource;
}(DataSource));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtTreeNodeOutletDirective = /** @class */ (function () {
    function NtTreeNodeOutletDirective(viewContainer) {
        this.viewContainer = viewContainer;
    }
    NtTreeNodeOutletDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntTreeNodeOutlet]',
                    providers: [
                        { provide: CdkTreeNodeOutlet, useExisting: NtTreeNodeOutletDirective }
                    ]
                },] },
    ];
    /** @nocollapse */
    NtTreeNodeOutletDirective.ctorParameters = function () { return [
        { type: ViewContainerRef }
    ]; };
    return NtTreeNodeOutletDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var NtTreeNodeDirective = /** @class */ (function (_super) {
    __extends(NtTreeNodeDirective, _super);
    function NtTreeNodeDirective(_elementRef, _tree, tabIndex) {
        var _this = _super.call(this, _elementRef, _tree) || this;
        _this._elementRef = _elementRef;
        _this._tree = _tree;
        _this._disabled = false;
        _this._tabIndex = 0;
        _this.role = 'treeitem';
        _this.tabIndex = Number(tabIndex) || 0;
        return _this;
    }
    Object.defineProperty(NtTreeNodeDirective.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtTreeNodeDirective.prototype, "tabIndex", {
        get: /**
         * @return {?}
         */
        function () { return this.disabled ? -1 : this._tabIndex; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._tabIndex = value != null ? value : 0; },
        enumerable: true,
        configurable: true
    });
    NtTreeNodeDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-tree-node, [nt-tree-node]',
                    exportAs: 'ntTreeNode',
                    inputs: ['disabled', 'tabIndex'],
                    host: {
                        'class': 'nt-tree-node'
                    },
                    providers: [{ provide: CdkTreeNode, useExisting: NtTreeNodeDirective }]
                },] },
    ];
    /** @nocollapse */
    NtTreeNodeDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: CdkTree },
        { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
    ]; };
    NtTreeNodeDirective.propDecorators = {
        role: [{ type: Input }]
    };
    return NtTreeNodeDirective;
}(CdkTreeNode));
/**
 * @template T
 */
var NtTreeNodeDefDirective = /** @class */ (function (_super) {
    __extends(NtTreeNodeDefDirective, _super);
    function NtTreeNodeDefDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtTreeNodeDefDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntTreeNodeDef]',
                    inputs: [
                        'when: ntTreeNodeDefWhen'
                    ],
                    providers: [{ provide: CdkTreeNodeDef, useExisting: NtTreeNodeDefDirective }]
                },] },
    ];
    NtTreeNodeDefDirective.propDecorators = {
        data: [{ type: Input, args: ['ntTreeNode',] }]
    };
    return NtTreeNodeDefDirective;
}(CdkTreeNodeDef));
/**
 * @template T
 */
var NtNestedTreeNodeDirective = /** @class */ (function (_super) {
    __extends(NtNestedTreeNodeDirective, _super);
    function NtNestedTreeNodeDirective(_elementRef, _tree, _differs, tabIndex) {
        var _this = _super.call(this, _elementRef, _tree, _differs) || this;
        _this._elementRef = _elementRef;
        _this._tree = _tree;
        _this._differs = _differs;
        _this._disabled = false;
        _this._tabIndex = 0;
        _this.tabIndex = Number(tabIndex) || 0;
        return _this;
    }
    Object.defineProperty(NtNestedTreeNodeDirective.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () { return this._disabled; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._disabled = coerceBooleanProperty(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtNestedTreeNodeDirective.prototype, "tabIndex", {
        get: /**
         * @return {?}
         */
        function () { return this.disabled ? -1 : this._tabIndex; },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) { this._tabIndex = value != null ? value : 0; },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NtNestedTreeNodeDirective.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        _super.prototype.ngAfterContentInit.call(this);
    };
    /**
     * @return {?}
     */
    NtNestedTreeNodeDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        _super.prototype.ngOnDestroy.call(this);
    };
    NtNestedTreeNodeDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'nt-nested-tree-node, [nt-nested-tree-node]',
                    exportAs: 'ntNestedTreeNode',
                    host: {
                        'class': 'nt-nested-tree-node',
                    },
                    inputs: ['disabled', 'tabIndex'],
                    providers: [
                        { provide: CdkNestedTreeNode, useExisting: NtNestedTreeNodeDirective },
                        { provide: CdkTreeNode, useExisting: NtNestedTreeNodeDirective }
                    ]
                },] },
    ];
    /** @nocollapse */
    NtNestedTreeNodeDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: CdkTree },
        { type: IterableDiffers },
        { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] }
    ]; };
    NtNestedTreeNodeDirective.propDecorators = {
        node: [{ type: Input, args: ['ntNestedTreeNode',] }],
        nodeOutlet: [{ type: ContentChildren, args: [NtTreeNodeOutletDirective,] }]
    };
    return NtNestedTreeNodeDirective;
}(CdkNestedTreeNode));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var NtTreeNodePaddingDirective = /** @class */ (function (_super) {
    __extends(NtTreeNodePaddingDirective, _super);
    function NtTreeNodePaddingDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtTreeNodePaddingDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntTreeNodePadding]',
                    providers: [{ provide: CdkTreeNodePadding, useExisting: NtTreeNodePaddingDirective }]
                },] },
    ];
    NtTreeNodePaddingDirective.propDecorators = {
        level: [{ type: Input, args: ['ntTreeNodePadding',] }],
        indent: [{ type: Input, args: ['ntTreeNodePaddingIndent',] }]
    };
    return NtTreeNodePaddingDirective;
}(CdkTreeNodePadding));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var NtTreeNodeToggleDirective = /** @class */ (function (_super) {
    __extends(NtTreeNodeToggleDirective, _super);
    function NtTreeNodeToggleDirective() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.recursive = false;
        return _this;
    }
    NtTreeNodeToggleDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ntTreeNodeToggle]',
                    host: {
                        '(click)': '_toggle($event)',
                    },
                    providers: [{ provide: CdkTreeNodeToggle, useExisting: NtTreeNodeToggleDirective }]
                },] },
    ];
    NtTreeNodeToggleDirective.propDecorators = {
        recursive: [{ type: Input, args: ['ntTreeNodeToggleRecursive',] }]
    };
    return NtTreeNodeToggleDirective;
}(CdkTreeNodeToggle));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var NtTreeComponent = /** @class */ (function (_super) {
    __extends(NtTreeComponent, _super);
    function NtTreeComponent() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NtTreeComponent.decorators = [
        { type: Component, args: [{
                    selector: 'nt-tree, [nt-tree]',
                    template: "<ng-container ntTreeNodeOutlet></ng-container>",
                    encapsulation: ViewEncapsulation.None,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [
                        { provide: CdkTree, useExisting: NtTreeComponent }
                    ],
                    host: {
                        'class': 'nt-tree'
                    }
                },] },
    ];
    return NtTreeComponent;
}(CdkTree));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var NT_TREE_DIRECTIVES = [
    NtNestedTreeNodeDirective,
    NtTreeNodeDefDirective,
    NtTreeNodePaddingDirective,
    NtTreeNodeToggleDirective,
    NtTreeComponent,
    NtTreeNodeDirective,
    NtTreeNodeOutletDirective
];
var NtTreeModule = /** @class */ (function () {
    function NtTreeModule() {
    }
    NtTreeModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CdkTreeModule, CommonModule],
                    exports: NT_TREE_DIRECTIVES,
                    declarations: NT_TREE_DIRECTIVES
                },] },
    ];
    return NtTreeModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtTreeNestedDataSource, NtTreeFlattener, NtTreeFlatDataSource, NtTreeNodeDirective, NtTreeNodeDefDirective, NtNestedTreeNodeDirective, NtTreeNodeOutletDirective, NtTreeNodePaddingDirective, NtTreeNodeToggleDirective, NtTreeComponent, NtTreeModule };
//# sourceMappingURL=tree.js.map
